@@ -579,7 +579,7 @@ bool Buffer::readDocument(Lexer & lex)
 		FileName const master_file = makeAbsPath(params().master,
 			   onlyPath(absFileName()));
 		if (isLyXFilename(master_file.absFilename())) {
-			Buffer * master = checkAndLoadLyXFile(master_file);
+			Buffer * master = checkAndLoadLyXFile(master_file, true);
 			d->parent_buffer = master;
 		}
 	}
@@ -676,6 +676,8 @@ bool Buffer::readFile(FileName const & filename)
 {
 	FileName fname(filename);
 
+	params().compressed = fname.isZippedFile();
+
 	// remove dummy empty par
 	paragraphs().clear();
 	Lexer lex;
@@ -1879,11 +1881,10 @@ void Buffer::updateMacros(DocIterator & it, DocIterator & scope) const
 			// is it an external file?
 			if (iit->inset->lyxCode() == INCLUDE_CODE) {
 				// get buffer of external file
-				InsetCommand const & inset
-					= static_cast<InsetCommand const &>(*iit->inset);
-				InsetCommandParams const & ip = inset.params();
+				InsetInclude const & inset
+					= static_cast<InsetInclude const &>(*iit->inset);
 				d->macro_lock = true;
-				Buffer * child = loadIfNeeded(*this, ip);
+				Buffer * child = inset.loadIfNeeded(*this);
 				d->macro_lock = false;
 				if (!child)
 					continue;
