@@ -15,60 +15,70 @@
 #ifndef BUFFERPARAMS_H
 #define BUFFERPARAMS_H
 
-#include "TextClass.h"
+#include "Font.h"
+#include "Citation.h"
 #include "paper.h"
 
 #include "insets/InsetQuotes.h"
 
 #include "support/copied_ptr.h"
-#include "support/types.h"
-
-#include "frontends/controllers/frontend_helpers.h"
 
+#include <list>
+#include <set>
 #include <vector>
 
-
 namespace lyx {
 
+namespace support { class FileName; }
+
 class AuthorList;
 class BranchList;
 class Bullet;
+class DocumentClass;
 class Encoding;
-class Lexer;
+class Language;
 class LatexFeatures;
+class LayoutFile;
+class LayoutFileIndex;
+class Lexer;
+class PDFOptions;
 class Spacing;
 class TexRow;
 class VSpace;
-class Language;
-
 
 /** Buffer parameters.
- *  This class contains all the parameters for this a buffer uses. Some
+ *  This class contains all the parameters for this buffer's use. Some
  *  work needs to be done on this class to make it nice. Now everything
  *  is in public.
  */
 class BufferParams {
 public:
 	///
-	enum PARSEP {
+	typedef std::list<std::string> LayoutModuleList;
+	///
+	enum ParagraphSeparation {
 		///
-		PARSEP_INDENT,
+		ParagraphIndentSeparation,
 		///
-		PARSEP_SKIP
+		ParagraphSkipSeparation
 	};
 	///
 	BufferParams();
-	~BufferParams();
 
 	/// get l10n translated to the buffers language
-	docstring const B_(std::string const & l10n) const;
+	docstring B_(std::string const & l10n) const;
 
 	/// read a header token, if unrecognised, return it or an unknown class name
-	std::string const readToken(Lexer & lex, std::string const & token);
+	std::string readToken(Lexer & lex,
+		std::string const & token, ///< token to read.
+		support::FileName const & filepath);
 
 	///
 	void writeFile(std::ostream &) const;
 
+	/// check what features are implied by the buffer parameters.
+	void validate(LaTeXFeatures &) const;
+
 	/** \returns true if the babel package is used (interogates
 	 *  the BufferParams and a LyXRC variable).
 	 *  This returned value can then be passed to the insets...
@@ -77,30 +87,67 @@ public:
 
 	///
 	void useClassDefaults();
-
 	///
 	bool hasClassDefaults() const;
 
 	///
 	VSpace const & getDefSkip() const;
-
 	///
 	void setDefSkip(VSpace const & vs);
 
-	/** Wether paragraphs are separated by using a indent like in
+	/** Whether paragraphs are separated by using a indent like in
 	 *  articles or by using a little skip like in letters.
 	 */
-	PARSEP paragraph_separation;
+	ParagraphSeparation paragraph_separation;
 	///
-	InsetQuotes::quote_language quotes_language;
+	InsetQuotes::QuoteLanguage quotes_language;
 	///
-	InsetQuotes::quote_times quotes_times;
+	InsetQuotes::QuoteTimes quotes_times;
 	///
 	std::string fontsize;
+	///Get the LayoutFile this document is using.
+	LayoutFile const * baseClass() const;
 	///
-	textclass_type textclass;
+	LayoutFileIndex const & baseClassID() const;
+	/// Set the LyX layout file this document is using.
+	/// NOTE: This does not call makeDocumentClass() to update the local 
+	/// DocumentClass. That needs to be done manually.
+	/// \param filename the name of the layout file
+	bool setBaseClass(std::string const & classname);
+	/// Adds the module information to the baseClass information to
+	/// create our local DocumentClass.
+	void makeDocumentClass();
+	/// Returns the DocumentClass currently in use: the BaseClass as modified
+	/// by modules.
+	DocumentClass const & documentClass() const;
+	/// \return A pointer to the DocumentClass currently in use: the BaseClass 
+	/// as modified by modules. 
+	DocumentClass const * documentClassPtr() const;
+	/// This bypasses the baseClass and sets the textClass directly.
+	/// Should be called with care and would be better not being here,
+	/// but it seems to be needed by CutAndPaste::putClipboard().
+	void setDocumentClass(DocumentClass const * const);
+	/// List of modules in use
+	LayoutModuleList const & getModules() const { return layoutModules_; }
+	/// List of default modules the user has removed
+	std::set<std::string> const & getRemovedModules() const 
+			{ return removedModules_; }
 	///
-	TextClass const & getTextClass() const;
+	/// Add a module to the list of modules in use. This checks only that the
+	/// module is not already in the list, so use moduleIsCompatible first if
+	/// you want to check for compatibility.
+	/// \return true if module was successfully added.
+	bool addLayoutModule(std::string const & modName);
+	/// checks to make sure module's requriements are satisfied, that it does
+	/// not conflict with already-present modules, isn't already loaded, etc.
+	bool moduleCanBeAdded(std::string const & modName) const;
+	///
+	void addRemovedModule(std::string const & modName) 
+			{ removedModules_.insert(modName); }
+	/// Clear the list
+	void clearLayoutModules() { layoutModules_.clear(); }
+	/// Clear the removed module list
+	void clearRemovedModules() { removedModules_.clear(); }
 
 	/// returns the main font for the buffer (document)
 	Font const getFont() const;
@@ -130,6 +177,8 @@ public:
 	std::string headsep;
 	///
 	std::string footskip;
+	///
+	std::string columnsep;
 
 	/* some LaTeX options */
 	/// The graphics driver
@@ -150,6 +199,8 @@ public:
 	int fontsSansScale;
 	/// the scale factor of the tt font
 	int fontsTypewriterScale;
+	/// the font used by the CJK command
+	std::string fontsCJK;
 	///
 	Spacing & spacing();
 	Spacing const & spacing() const;
@@ -184,7 +235,13 @@ public:
 	///
 	std::string preamble;
 	///
+	std::string local_layout;
+	///
 	std::string options;
+	/// use the class options defined in the layout?
+	bool use_default_options;
+	///
+	std::string master;
 	///
 	std::string float_placement;
 	///
@@ -192,7 +249,7 @@ public:
 	/// parameters for the listings package
 	std::string listings_params;
 	///
-	TextClass::PageSides sides;
+	PageSides sides;
 	///
 	std::string pagestyle;
 	/// \param index should lie in the range 0 <= \c index <= 3.
@@ -201,16 +258,6 @@ public:
 	/// \param index should lie in the range 0 <= \c index <= 3.
 	Bullet & user_defined_bullet(size_type index);
 	Bullet const & user_defined_bullet(size_type index) const;
-	///
-	void readPreamble(Lexer &);
-	///
-	void readLanguage(Lexer &);
-	///
-	void readGraphicsDriver(Lexer &);
-	///
-	void readBullets(Lexer &);
-	///
-	void readBulletsLaTeX(Lexer &);
 
 	/// Whether to load a package such as amsmath or esint.
 	/// The enum values must not be changed (file format!)
@@ -236,8 +283,6 @@ public:
 	 *  for instance, they may differ for DVI and PDF generation)
 	 */
 	bool outputChanges;
-	/// Time ago we agreed that this was a buffer property [ale990407]
-	std::string parentname;
 	///
 	bool compressed;
 
@@ -253,7 +298,7 @@ public:
 	 *  purpose for which the paper size is needed, since they
 	 *  support different subsets of paper sizes.
 	*/
-	enum Papersize_Purpose {
+	enum PapersizePurpose {
 		///
 		DVIPS,
 		///
@@ -262,9 +307,9 @@ public:
 		XDVI
 	};
 	///
-	std::string const paperSizeName(Papersize_Purpose const & purpose) const;
+	std::string paperSizeName(PapersizePurpose purpose) const;
 	/// set up if and how babel is called
-	std::string const babelCall(std::string const & lang_opts) const;
+	std::string babelCall(std::string const & lang_opts) const;
 	/// handle inputenc etc.
 	void writeEncodingPreamble(odocstream & os, LaTeXFeatures & features,
 					      TexRow & texrow) const;
@@ -273,15 +318,57 @@ public:
 				     std::string const & sf, std::string const & tt,
 				     bool const & sc, bool const & osf,
 				     int const & sfscale, int const & ttscale) const;
-	/// path of the current buffer
-	std::string filepath;
-	/// get the appropriate cite engine (natbib handling)
-	biblio::CiteEngine getEngine() const;
 
+	/// get the appropriate cite engine (natbib handling)
+	CiteEngine citeEngine() const;
 	///
-	void setCiteEngine(biblio::CiteEngine const);
+	void setCiteEngine(CiteEngine const);
+
+	/// options for pdf output
+	PDFOptions & pdfoptions();
+	PDFOptions const & pdfoptions() const;
 
 private:
+	///
+	void readPreamble(Lexer &);
+	///
+	void readLocalLayout(Lexer &);
+	///
+	void readLanguage(Lexer &);
+	///
+	void readGraphicsDriver(Lexer &);
+	///
+	void readBullets(Lexer &);
+	///
+	void readBulletsLaTeX(Lexer &);
+	///
+	void readModules(Lexer &);
+	///
+	void readRemovedModules(Lexer &);
+	/// Called when the document class changes. Removes modules
+	/// excluded by, provided by, etc, the document class.
+	/// \return true if modules were consistent, false if changes had
+	/// to be made.
+	bool removeBadModules();
+	/// Adds default modules, if they're addable.
+	void addDefaultModules();
+	/// checks for consistency among modules: makes sure requirements
+	/// are met, no modules exclude one another, etc, and resolves any
+	/// such conflicts, leaving us with a consistent collection.
+	/// \return true if modules were consistent, false if changes had
+	/// to be made.
+	bool checkModuleConsistency();
+
+	/// for use with natbib
+	CiteEngine cite_engine_;
+	///
+	DocumentClass * doc_class_;
+	/// 
+	LayoutModuleList layoutModules_;
+	/// this is for modules that are required by the document class but that
+	/// the user has chosen not to use
+	std::set<std::string> removedModules_;
+
 	/** Use the Pimpl idiom to hide those member variables that would otherwise
 	 *  drag in other header files.
 	 */
@@ -293,8 +380,6 @@ private:
 	};
 	support::copied_ptr<Impl, MemoryTraits> pimpl_;
 
-	///
-	biblio::CiteEngine cite_engine_;
 };
 
 } // namespace lyx
