@@ -18,40 +18,33 @@
 #include "Lexer.h"
 #include "LyXRC.h"
 #include "Mover.h"
-#include "debug.h"
 
 #include "support/convert.h"
+#include "support/debug.h"
 #include "support/filetools.h"
-#include "support/lyxlib.h"
 #include "support/lyxtime.h"
 #include "support/Package.h"
 
+#include "support/lassert.h"
 #include <boost/crc.hpp>
-#include <boost/filesystem/operations.hpp>
-#include <boost/current_function.hpp>
 
+#include <algorithm>
 #include <fstream>
 #include <iomanip>
 #include <map>
 #include <sstream>
 
-using lyx::support::addName;
-
-using std::string;
-
-namespace fs = boost::filesystem;
+using namespace std;
+using namespace lyx::support;
 
 namespace lyx {
 
-using support::doesFileExist;
-using support::FileName;
-
 namespace {
 
 unsigned long do_crc(string const & s)
 {
 	boost::crc_32_type crc;
-	crc = std::for_each(s.begin(), s.end(), crc);
+	crc = for_each(s.begin(), s.end(), crc);
 	return crc.checksum();
 }
 
@@ -66,14 +59,13 @@ public:
 		  time_t t, unsigned long c)
 		: timestamp(t), checksum(c)
 	{
-		std::ostringstream os;
-		os << std::setw(10) << std::setfill('0') << do_crc(orig_from.absFilename())
+		ostringstream os;
+		os << setw(10) << setfill('0') << do_crc(orig_from.absFilename())
 		   << '-' << to_format;
 		cache_name = FileName(addName(cache_dir.absFilename(), os.str()));
-		LYXERR(Debug::FILES) << "Add file cache item " << orig_from
+		LYXERR(Debug::FILES, "Add file cache item " << orig_from
 				     << ' ' << to_format << ' ' << cache_name
-				     << ' ' << timestamp << ' ' << checksum
-				     << '.' << std::endl;
+				     << ' ' << long(timestamp) << ' ' << checksum << '.');
 	}
 	~CacheItem() {}
 	FileName cache_name;
@@ -87,7 +79,7 @@ public:
 /** The cache contains one item per orig file and target format, so use a
  *  nested map to find the cache item quickly by filename and format.
  */
-typedef std::map<string, CacheItem> FormatCacheType;
+typedef map<string, CacheItem> FormatCacheType;
 class FormatCache {
 public:
 	/// Format of the source file
@@ -95,7 +87,7 @@ public:
 	/// Cache target format -> item to quickly find the item by format
 	FormatCacheType cache;
 };
-typedef std::map<FileName, FormatCache> CacheType;
+typedef map<FileName, FormatCache> CacheType;
 
 
 class ConverterCache::Impl {
@@ -114,8 +106,8 @@ void ConverterCache::Impl::readIndex()
 {
 	time_t const now = current_time();
 	FileName const index(addName(cache_dir.absFilename(), "index"));
-	std::ifstream is(index.toFilesystemEncoding().c_str());
-	Lexer lex(0, 0);
+	ifstream is(index.toFilesystemEncoding().c_str());
+	Lexer lex;
 	lex.setStream(is);
 	while (lex.isOK()) {
 		if (!lex.next(true))
@@ -136,31 +128,28 @@ void ConverterCache::Impl::readIndex()
 		CacheItem item(orig_from_name, to_format, timestamp, checksum);
 
 		// Don't cache files that do not exist anymore
-		if (!doesFileExist(orig_from_name)) {
-			LYXERR(Debug::FILES) << "Not caching file `"
-				<< orig_from << "' (does not exist anymore)."
-				<< std::endl;
-			support::unlink(item.cache_name);
+		if (!orig_from_name.exists()) {
+			LYXERR(Debug::FILES, "Not caching file `"
+				<< orig_from << "' (does not exist anymore).");
+			item.cache_name.removeFile();
 			continue;
 		}
 
 		// Don't add items that are not in the cache anymore
 		// This can happen if two instances of LyX are running
 		// at the same time and update the index file independantly.
-		if (!doesFileExist(item.cache_name)) {
-			LYXERR(Debug::FILES) << "Not caching file `"
-				<< orig_from
-				<< "' (cached copy does not exist anymore)."
-				<< std::endl;
+		if (!item.cache_name.exists()) {
+			LYXERR(Debug::FILES, "Not caching file `" << orig_from
+				<< "' (cached copy does not exist anymore).");
 			continue;
 		}
 
 		// Delete the cached file if it is too old
-		if (difftime(now, fs::last_write_time(item.cache_name.toFilesystemEncoding())) >
-		    lyxrc.converter_cache_maxage) {
-			LYXERR(Debug::FILES) << "Not caching file `"
-				<< orig_from << "' (too old)." << std::endl;
-			support::unlink(item.cache_name);
+		if (difftime(now, item.cache_name.lastModified())
+				> lyxrc.converter_cache_maxage) {
+			LYXERR(Debug::FILES, "Not caching file `"
+				<< orig_from << "' (too old).");
+			item.cache_name.removeFile();
 			continue;
 		}
 
@@ -177,9 +166,9 @@ void ConverterCache::Impl::readIndex()
 void ConverterCache::Impl::writeIndex()
 {
 	FileName const index(addName(cache_dir.absFilename(), "index"));
-	std::ofstream os(index.toFilesystemEncoding().c_str());
+	ofstream os(index.toFilesystemEncoding().c_str());
 	os.close();
-	if (!lyx::support::chmod(index, 0600))
+	if (!index.changePermission(0600))
 		return;
 	os.open(index.toFilesystemEncoding().c_str());
 	CacheType::iterator it1 = cache.begin();
@@ -214,6 +203,26 @@ CacheItem * ConverterCache::Impl::find(FileName const & from,
 }
 
 
+/////////////////////////////////////////////////////////////////////
+//
+// ConverterCache
+//
+/////////////////////////////////////////////////////////////////////
+
+ConverterCache::ConverterCache()
+	: pimpl_(new Impl)
+{}
+
+
+ConverterCache::~ConverterCache()
+{
+	if (!lyxrc.use_converter_cache)
+		return;
+	pimpl_->writeIndex();
+	delete pimpl_;
+}
+
+
 ConverterCache & ConverterCache::get()
 {
 	// Now return the cache
@@ -228,95 +237,81 @@ void ConverterCache::init()
 		return;
 	// We do this here and not in the constructor because package() gets
 	// initialized after all static variables.
-	cache_dir = FileName(addName(support::package().user_support().absFilename(), "cache"));
-	if (!doesFileExist(cache_dir))
-		if (support::mkdir(cache_dir, 0700) != 0) {
+	cache_dir = FileName(addName(package().user_support().absFilename(), "cache"));
+	if (!cache_dir.exists())
+		if (!cache_dir.createDirectory(0700)) {
 			lyxerr << "Could not create cache directory `"
-			       << cache_dir << "'." << std::endl;
+			       << cache_dir << "'." << endl;
 			exit(EXIT_FAILURE);
 		}
 	get().pimpl_->readIndex();
 }
 
 
-ConverterCache::ConverterCache()
-	: pimpl_(new Impl)
-{}
-
-
-ConverterCache::~ConverterCache()
-{
-	if (!lyxrc.use_converter_cache)
-		return;
-	pimpl_->writeIndex();
-}
-
-
 void ConverterCache::add(FileName const & orig_from, string const & to_format,
 		FileName const & converted_file) const
 {
 	if (!lyxrc.use_converter_cache || orig_from.empty() ||
 	    converted_file.empty())
 		return;
-	LYXERR(Debug::FILES) << BOOST_CURRENT_FUNCTION << ' ' << orig_from
-			     << ' ' << to_format << ' ' << converted_file
-			     << std::endl;
+	LYXERR(Debug::FILES, ' ' << orig_from
+			     << ' ' << to_format << ' ' << converted_file);
 
 	// FIXME: Should not hardcode this (see bug 3819 for details)
 	if (to_format == "pstex") {
-		FileName const converted_eps(support::changeExtension(converted_file.absFilename(), "eps"));
+		FileName const converted_eps(changeExtension(converted_file.absFilename(), "eps"));
 		add(orig_from, "eps", converted_eps);
 	} else if (to_format == "pdftex") {
-		FileName const converted_pdf(support::changeExtension(converted_file.absFilename(), "pdf"));
+		FileName const converted_pdf(changeExtension(converted_file.absFilename(), "pdf"));
 		add(orig_from, "pdf", converted_pdf);
 	}
 
 	// Is the file in the cache already?
 	CacheItem * item = pimpl_->find(orig_from, to_format);
 
-	time_t const timestamp = fs::last_write_time(orig_from.toFilesystemEncoding());
+	time_t const timestamp = orig_from.lastModified();
 	Mover const & mover = getMover(to_format);
 	if (item) {
-		LYXERR(Debug::FILES) << "ConverterCache::add(" << orig_from << "):\n"
-					"The file is already in the cache."
-				     << std::endl;
+		LYXERR(Debug::FILES, "ConverterCache::add(" << orig_from << "):\n"
+					"The file is already in the cache.");
 		// First test for timestamp
 		if (timestamp == item->timestamp) {
-			LYXERR(Debug::FILES) << "Same timestamp."
-					     << std::endl;
+			LYXERR(Debug::FILES, "Same timestamp.");
 			return;
-		} else {
-			// Maybe the contents is still the same?
-			item->timestamp = timestamp;
-			unsigned long const checksum = support::sum(orig_from);
-			if (checksum == item->checksum) {
-				LYXERR(Debug::FILES) << "Same checksum."
-						     << std::endl;
-				return;
-			}
-			item->checksum = checksum;
 		}
+		// Maybe the contents is still the same?
+		item->timestamp = timestamp;
+		unsigned long const checksum = orig_from.checksum();
+		if (checksum == item->checksum) {
+			LYXERR(Debug::FILES, "Same checksum.");
+			return;
+		}
+		item->checksum = checksum;
 		if (!mover.copy(converted_file, item->cache_name,
-		                support::onlyFilename(item->cache_name.absFilename()), 0600))
-			LYXERR(Debug::FILES) << "ConverterCache::add("
-					     << orig_from << "):\n"
-						"Could not copy file."
-					     << std::endl;
+		              onlyFilename(item->cache_name.absFilename()))) {
+			LYXERR(Debug::FILES, "Could not copy file " << orig_from << " to "
+				<< item->cache_name);
+		} else if (!item->cache_name.changePermission(0600)) {
+			LYXERR(Debug::FILES, "Could not change file mode"
+				<< item->cache_name);
+		}
 	} else {
 		CacheItem new_item(orig_from, to_format, timestamp,
-				support::sum(orig_from));
+				orig_from.checksum());
 		if (mover.copy(converted_file, new_item.cache_name,
-		               support::onlyFilename(new_item.cache_name.absFilename()), 0600)) {
+		              onlyFilename(new_item.cache_name.absFilename()))) {
+			if (!new_item.cache_name.changePermission(0600)) {
+				LYXERR(Debug::FILES, "Could not change file mode"
+					<< new_item.cache_name);
+			}
 			FormatCache & format_cache = pimpl_->cache[orig_from];
 			if (format_cache.from_format.empty())
 				format_cache.from_format =
 					formats.getFormatFromFile(orig_from);
 			format_cache.cache[to_format] = new_item;
 		} else
-			LYXERR(Debug::FILES) << "ConverterCache::add("
-					     << orig_from << "):\n"
-						"Could not copy file."
-				     << std::endl;
+			LYXERR(Debug::FILES, "ConverterCache::add(" << orig_from << "):\n"
+						"Could not copy file.");
 	}
 }
 
@@ -326,8 +321,7 @@ void ConverterCache::remove(FileName const & orig_from,
 {
 	if (!lyxrc.use_converter_cache || orig_from.empty())
 		return;
-	LYXERR(Debug::FILES) << BOOST_CURRENT_FUNCTION << ' ' << orig_from
-			     << ' ' << to_format << std::endl;
+	LYXERR(Debug::FILES, orig_from << ' ' << to_format);
 
 	CacheType::iterator const it1 = pimpl_->cache.find(orig_from);
 	if (it1 == pimpl_->cache.end())
@@ -358,25 +352,25 @@ void ConverterCache::remove_all(string const & from_format,
 		FormatCacheType::iterator it2 = format_cache.begin();
 		while (it2 != format_cache.end()) {
 			if (it2->first == to_format) {
-				LYXERR(Debug::FILES)
-					<< "Removing file cache item "
-					<< it1->first
-					<< ' ' << to_format << std::endl;
-				support::unlink(it2->second.cache_name);
+				LYXERR(Debug::FILES, "Removing file cache item "
+					<< it1->first << ' ' << to_format);
+				it2->second.cache_name.removeFile();
 				format_cache.erase(it2);
 				// Have to start over again since items in a
 				// map are not ordered
 				it2 = format_cache.begin();
-			} else
+			} else {
 				++it2;
+			}
 		}
 		if (format_cache.empty()) {
 			pimpl_->cache.erase(it1);
 			// Have to start over again since items in a map are
 			// not ordered
 			it1 = pimpl_->cache.begin();
-		} else
+		} else {
 			++it1;
+		}
 	}
 	pimpl_->writeIndex();
 }
@@ -387,25 +381,24 @@ bool ConverterCache::inCache(FileName const & orig_from,
 {
 	if (!lyxrc.use_converter_cache || orig_from.empty())
 		return false;
-	LYXERR(Debug::FILES) << BOOST_CURRENT_FUNCTION << ' ' << orig_from
-			     << ' ' << to_format << std::endl;
+	LYXERR(Debug::FILES, orig_from << ' ' << to_format);
 
 	CacheItem * const item = pimpl_->find(orig_from, to_format);
 	if (!item) {
-		LYXERR(Debug::FILES) << "not in cache." << std::endl;
+		LYXERR(Debug::FILES, "not in cache.");
 		return false;
 	}
-	time_t const timestamp = fs::last_write_time(orig_from.toFilesystemEncoding());
+	time_t const timestamp = orig_from.lastModified();
 	if (item->timestamp == timestamp) {
-		LYXERR(Debug::FILES) << "identical timestamp." << std::endl;
+		LYXERR(Debug::FILES, "identical timestamp.");
 		return true;
 	}
-	if (item->checksum == support::sum(orig_from)) {
+	if (item->checksum == orig_from.checksum()) {
 		item->timestamp = timestamp;
-		LYXERR(Debug::FILES) << "identical checksum." << std::endl;
+		LYXERR(Debug::FILES, "identical checksum.");
 		return true;
 	}
-	LYXERR(Debug::FILES) << "in cache, but too old." << std::endl;
+	LYXERR(Debug::FILES, "in cache, but too old.");
 	return false;
 }
 
@@ -413,11 +406,10 @@ bool ConverterCache::inCache(FileName const & orig_from,
 FileName const & ConverterCache::cacheName(FileName const & orig_from,
 		string const & to_format) const
 {
-	LYXERR(Debug::FILES) << BOOST_CURRENT_FUNCTION << ' ' << orig_from
-			     << ' ' << to_format << std::endl;
+	LYXERR(Debug::FILES, orig_from << ' ' << to_format);
 
 	CacheItem * const item = pimpl_->find(orig_from, to_format);
-	BOOST_ASSERT(item);
+	LASSERT(item, /**/);
 	return item->cache_name;
 }
 
@@ -427,25 +419,24 @@ bool ConverterCache::copy(FileName const & orig_from, string const & to_format,
 {
 	if (!lyxrc.use_converter_cache || orig_from.empty() || dest.empty())
 		return false;
-	LYXERR(Debug::FILES) << BOOST_CURRENT_FUNCTION << ' ' << orig_from
-			     << ' ' << to_format << ' ' << dest << std::endl;
+	LYXERR(Debug::FILES, orig_from << ' ' << to_format << ' ' << dest);
 
 	// FIXME: Should not hardcode this (see bug 3819 for details)
 	if (to_format == "pstex") {
-		FileName const dest_eps(support::changeExtension(dest.absFilename(), "eps"));
+		FileName const dest_eps(changeExtension(dest.absFilename(), "eps"));
 		if (!copy(orig_from, "eps", dest_eps))
 			return false;
 	} else if (to_format == "pdftex") {
-		FileName const dest_pdf(support::changeExtension(dest.absFilename(), "pdf"));
+		FileName const dest_pdf(changeExtension(dest.absFilename(), "pdf"));
 		if (!copy(orig_from, "pdf", dest_pdf))
 			return false;
 	}
 
 	CacheItem * const item = pimpl_->find(orig_from, to_format);
-	BOOST_ASSERT(item);
+	LASSERT(item, /**/);
 	Mover const & mover = getMover(to_format);
 	return mover.copy(item->cache_name, dest,
-	                  support::onlyFilename(dest.absFilename()));
+	                  onlyFilename(dest.absFilename()));
 }
 
 } // namespace lyx
