@@ -14,30 +14,32 @@
 
 #include "Encoding.h"
 
-#include "debug.h"
+#include "Buffer.h"
+#include "InsetIterator.h"
 #include "LaTeXFeatures.h"
 #include "Lexer.h"
 #include "LyXRC.h"
 
+#include "support/debug.h"
 #include "support/FileName.h"
 #include "support/lstrings.h"
 #include "support/unicode.h"
 
+#include <boost/cstdint.hpp>
+
 #include <sstream>
 
-#ifndef CXX_GLOBAL_CSTD
-using std::strtol;
-#endif
-using std::endl;
-using std::string;
-
+using namespace std;
+using namespace lyx::support;
 
 namespace lyx {
 
-using support::FileName;
-
 Encodings encodings;
 
+Encodings::MathCommandSet Encodings::mathcmd;
+Encodings::TextCommandSet Encodings::textcmd;
+Encodings::MathSymbolSet  Encodings::mathsym;
+
 namespace {
 
 char_type arabic_table[172][4] = {
@@ -226,27 +228,37 @@ char_type const arabic_end = 0x06cc;
 
 /// Information about a single UCS4 character
 struct CharInfo {
-	/// LaTeX command for this character
-	docstring command;
-	/// Needed LaTeX preamble (or feature)
-	string preamble;
+	/// LaTeX command (text mode) for this character
+	docstring textcommand;
+	/// LaTeX command (math mode) for this character
+	docstring mathcommand;
+	/// Needed LaTeX preamble (or feature) for text mode
+	string textpreamble;
+	/// Needed LaTeX preamble (or feature) for math mode
+	string mathpreamble;
 	/// Is this a combining character?
 	bool combining;
-	/// Is \c preamble a feature known by LaTeXFeatures, or a raw LaTeX
+	/// Is \c textpreamble a feature known by LaTeXFeatures, or a raw LaTeX
 	/// command?
-	bool feature;
+	bool textfeature;
+	/// Is \c mathpreamble a feature known by LaTeXFeatures, or a raw LaTeX
+	/// command?
+	bool mathfeature;
 	/// Always force the LaTeX command, even if the encoding contains
 	/// this character?
 	bool force;
 };
 
 
-typedef std::map<char_type, CharInfo> CharInfoMap;
+typedef map<char_type, CharInfo> CharInfoMap;
 CharInfoMap unicodesymbols;
 
 typedef std::set<char_type> CharSet;
 CharSet forced;
 
+typedef std::set<char_type> MathAlphaSet;
+MathAlphaSet mathalpha;
+
 
 /// The highest code point in UCS4 encoding (1<<20 + 1<<16)
 char_type const max_ucs4 = 0x110000;
@@ -266,9 +278,9 @@ const char * EncodingException::what() const throw()
 }
 
 
-Encoding::Encoding(string const & n, string const & l, string const & i,
-		   bool f, Encoding::Package p)
-	: Name_(n), LatexName_(l), iconvName_(i), fixedwidth_(f), package_(p)
+Encoding::Encoding(string const & n, string const & l, string const & g,
+		   string const & i, bool f, Encoding::Package p)
+	: name_(n), latexName_(l), guiName_(g), iconvName_(i), fixedwidth_(f), package_(p)
 {
 	if (n == "ascii") {
 		// ASCII can encode 128 code points and nothing else
@@ -286,6 +298,9 @@ Encoding::Encoding(string const & n, string const & l, string const & i,
 
 void Encoding::init() const
 {
+	if (complete_)
+		return;
+
 	start_encodable_ = 0;
 	// temporarily switch off lyxerr, since we will generate iconv errors
 	lyxerr.disable();
@@ -293,14 +308,14 @@ void Encoding::init() const
 		// We do not need to check all UCS4 code points, it is enough
 		// if we check all 256 code points of this encoding.
 		for (unsigned short j = 0; j < 256; ++j) {
-			char const c = j;
-			std::vector<char_type> const ucs4 = eightbit_to_ucs4(&c, 1, iconvName_);
-			if (ucs4.size() == 1) {
-				char_type const c = ucs4[0];
-				CharInfoMap::const_iterator const it = unicodesymbols.find(c);
-				if (it == unicodesymbols.end() || !it->second.force)
-					encodable_.insert(c);
-			}
+			char const c = char(j);
+			vector<char_type> const ucs4 = eightbit_to_ucs4(&c, 1, iconvName_);
+			if (ucs4.size() != 1)
+				continue;
+			char_type const uc = ucs4[0];
+			CharInfoMap::const_iterator const it = unicodesymbols.find(uc);
+			if (it == unicodesymbols.end() || !it->second.force)
+				encodable_.insert(uc);
 		}
 	} else {
 		// We do not know how many code points this encoding has, and
@@ -308,7 +323,7 @@ void Encoding::init() const
 		// therefore we need to check all UCS4 code points.
 		// This is expensive!
 		for (char_type c = 0; c < max_ucs4; ++c) {
-			std::vector<char> const eightbit = ucs4_to_eightbit(&c, 1, iconvName_);
+			vector<char> const eightbit = ucs4_to_eightbit(&c, 1, iconvName_);
 			if (!eightbit.empty()) {
 				CharInfoMap::const_iterator const it = unicodesymbols.find(c);
 				if (it == unicodesymbols.end() || !it->second.force)
@@ -327,45 +342,236 @@ void Encoding::init() const
 }
 
 
-docstring const Encoding::latexChar(char_type c) const
+docstring Encoding::latexChar(char_type c, bool for_mathed) const
 {
 	// assure the used encoding is properly initialized
-	if (!complete_)
-		init();
-	BOOST_ASSERT(complete_);
+	init();
 
 	if (iconvName_ == "UTF-8" && package_ == none)
 		return docstring(1, c);
 	if (c < start_encodable_ && !encodings.isForced(c))
 		return docstring(1, c);
-	if (encodable_.find(c) == encodable_.end()) {
-		// c cannot be encoded in this encoding
-		CharInfoMap::const_iterator const it = unicodesymbols.find(c);
-		if (it == unicodesymbols.end())
-			throw EncodingException(c);
-		else
-			return it->second.command;
-	}
-	return docstring(1, c);
+	if (encodable_.find(c) != encodable_.end())
+		return docstring(1, c);
+	if (for_mathed)
+		return docstring();
+
+	// c cannot (or should not) be encoded in this encoding
+	CharInfoMap::const_iterator const it = unicodesymbols.find(c);
+	if (it == unicodesymbols.end())
+		throw EncodingException(c);
+	// at least one of mathcommand and textcommand is nonempty
+	if (it->second.textcommand.empty())
+		return "\\ensuremath{" + it->second.mathcommand + '}';
+	return it->second.textcommand;
 }
 
 
-void Encodings::validate(char_type c, LaTeXFeatures & features)
+vector<char_type> Encoding::symbolsList() const
+{
+	// assure the used encoding is properly initialized
+	init();
+
+	// first all encodable characters
+	vector<char_type> symbols(encodable_.begin(), encodable_.end());
+	// add those below start_encodable_
+	for (char_type c = 0; c < start_encodable_; ++c)
+		symbols.push_back(c);
+	// now the ones from the unicodesymbols file
+	CharInfoMap::const_iterator const end = unicodesymbols.end();
+	CharInfoMap::const_iterator it = unicodesymbols.begin();
+	for (; it != end; ++it)
+		symbols.push_back(it->first);
+	return symbols;
+}
+
+
+bool Encodings::latexMathChar(char_type c, bool mathmode,
+			Encoding const * encoding, docstring & command)
+{
+	if (encoding)
+		command = encoding->latexChar(c, true);
+
+	CharInfoMap::const_iterator const it = unicodesymbols.find(c);
+	if (it == unicodesymbols.end()) {
+		if (!encoding || command.empty())
+			throw EncodingException(c);
+		if (mathmode)
+			addMathSym(c);
+		return false;
+	}
+	// at least one of mathcommand and textcommand is nonempty
+	bool use_math = (mathmode && !it->second.mathcommand.empty()) ||
+			(!mathmode && it->second.textcommand.empty());
+	if (use_math) {
+		command = it->second.mathcommand;
+		addMathCmd(c);
+	} else {
+		if (!encoding || command.empty()) {
+			command = it->second.textcommand;
+			addTextCmd(c);
+		}
+		if (mathmode)
+			addMathSym(c);
+	}
+	return use_math;
+}
+
+
+char_type Encodings::fromLaTeXCommand(docstring const & cmd, bool & combining)
+{
+	CharInfoMap::const_iterator const end = unicodesymbols.end();
+	CharInfoMap::const_iterator it = unicodesymbols.begin();
+	for (combining = false; it != end; ++it) {
+		docstring const math = it->second.mathcommand;
+		docstring const text = it->second.textcommand;
+		if (math == cmd || text == cmd) {
+			combining = it->second.combining;
+			return it->first;
+		}
+	}
+	return 0;
+}
+
+
+docstring Encodings::fromLaTeXCommand(docstring const & cmd, docstring & rem)
+{
+	docstring symbols;
+	size_t i = 0;
+	size_t const cmdend = cmd.size();
+	CharInfoMap::const_iterator const uniend = unicodesymbols.end();
+	for (size_t j = 0; j < cmdend; ++j) {
+		// Also get the char after a backslash
+		if (j + 1 < cmdend && cmd[j] == '\\')
+			++j;
+		// If a macro argument follows, get it, too
+		if (j + 1 < cmdend && cmd[j + 1] == '{') {
+			size_t k = j + 1;
+			int count = 1;
+			while (k < cmdend && count && k != docstring::npos) {
+				k = cmd.find_first_of(from_ascii("{}"), k + 1);
+				if (cmd[k] == '{')
+					++count;
+				else
+					--count;
+			}
+			if (k != docstring::npos)
+				j = k;
+		}
+		// Start with this substring and try augmenting it when it is
+		// the prefix of some command in the unicodesymbols file
+		docstring const subcmd = cmd.substr(i, j - i + 1);
+
+		CharInfoMap::const_iterator it = unicodesymbols.begin();
+		size_t unicmd_size = 0;
+		char_type c = 0;
+		for (; it != uniend; ++it) {
+			docstring const math = it->second.mathcommand;
+			docstring const text = it->second.textcommand;
+			size_t cur_size = max(math.size(), text.size());
+			// The current math or text unicode command cannot
+			// match, or we already matched a longer one
+			if (cur_size < subcmd.size() || cur_size <= unicmd_size)
+				continue;
+
+			docstring tmp = subcmd;
+			size_t k = j;
+			while (prefixIs(math, tmp) || prefixIs(text, tmp)) {
+				++k;
+				if (k >= cmdend || cur_size <= tmp.size())
+					break;
+				tmp += cmd[k];
+			}
+			// No match
+			if (k == j)
+				continue;
+
+			// The last added char caused a mismatch, because
+			// we didn't exhaust the chars in cmd and didn't
+			// exceed the maximum size of the current unicmd
+			if (k < cmdend && cur_size > tmp.size())
+				tmp.resize(tmp.size() - 1);
+
+			// If this is an exact match, we found a (longer)
+			// matching command in the unicodesymbols file
+			if (math == tmp || text == tmp) {
+				c = it->first;
+				j = k - 1;
+				i = j + 1;
+				unicmd_size = cur_size;
+			}
+		}
+		if (unicmd_size)
+			symbols += c;
+		else if (j + 1 == cmdend)
+			// No luck. Return what remains
+			rem = cmd.substr(i);
+	}
+	return symbols;
+}
+
+
+void Encodings::initUnicodeMath(Buffer const & buffer)
+{
+	mathcmd.clear();
+	textcmd.clear();
+	mathsym.clear();
+
+	Inset & inset = buffer.inset();
+	InsetIterator it = inset_iterator_begin(inset);
+	InsetIterator const end = inset_iterator_end(inset);
+
+	for (; it != end; ++it)
+		it->initUnicodeMath();
+}
+
+
+void Encodings::validate(char_type c, LaTeXFeatures & features, bool for_mathed)
 {
 	CharInfoMap::const_iterator const it = unicodesymbols.find(c);
-	if (it != unicodesymbols.end() && !it->second.preamble.empty()) {
-		if (it->second.feature)
-			features.require(it->second.preamble);
-		else
-			features.addPreambleSnippet(it->second.preamble);
+	if (it != unicodesymbols.end()) {
+		// In mathed, c could be used both in textmode and mathmode
+		bool const use_math = (for_mathed && isMathCmd(c)) ||
+		                      (!for_mathed && it->second.textcommand.empty());
+		bool const use_text = (for_mathed && isTextCmd(c)) ||
+		                      (!for_mathed && !it->second.textcommand.empty());
+		if (use_math) {
+			if (!it->second.mathpreamble.empty()) {
+				if (it->second.mathfeature) {
+					string feats = it->second.mathpreamble;
+					while (!feats.empty()) {
+						string feat;
+						feats = split(feats, feat, ',');
+						features.require(feat);
+					}
+				} else
+					features.addPreambleSnippet(it->second.mathpreamble);
+			}
+		}
+		if (use_text) {
+			if (!it->second.textpreamble.empty()) {
+				if (it->second.textfeature) {
+					string feats = it->second.textpreamble;
+					while (!feats.empty()) {
+						string feat;
+						feats = split(feats, feat, ',');
+						features.require(feat);
+					}
+				} else
+					features.addPreambleSnippet(it->second.textpreamble);
+			}
+		}
+	}
+	if (for_mathed && isMathSym(c)) {
+		features.require("relsize");
+		features.require("lyxmathsym");
 	}
 }
 
 
-bool Encodings::isComposeChar_hebrew(char_type c)
+bool Encodings::isHebrewComposeChar(char_type c)
 {
-	return c <= 0x05c2 && c >= 0x05b0 &&
-	       c != 0x05be && c != 0x05c0;
+	return c <= 0x05c2 && c >= 0x05b0 && c != 0x05be && c != 0x05c0;
 }
 
 
@@ -373,36 +579,30 @@ bool Encodings::isComposeChar_hebrew(char_type c)
 // they are hamza, alef_madda, alef_hamza, waw_hamza, alef_hamza_under,
 // alef, tah_marbota, dal, thal, rah, zai, wow, alef_maksoura
 
-bool Encodings::is_arabic_special(char_type c)
+bool Encodings::isArabicSpecialChar(char_type c)
 {
-	return (c >= 0x0621 && c <= 0x0625) ||
-		c == 0x0627 || c == 0x0629  ||
-		c == 0x062f || c == 0x0648  ||
-	       (c >= 0x0630 && c <= 0x0632) ||
-       	        c == 0x0649 || c == 0x0698;
+	return (c >= 0x0621 && c <= 0x0625) || (c >= 0x0630 && c <= 0x0632)
+		|| c == 0x0627 || c == 0x0629 || c == 0x062f || c == 0x0648
+		|| c == 0x0649 || c == 0x0698;
 }
 
 
-bool Encodings::isComposeChar_arabic(char_type c)
+bool Encodings::isArabicComposeChar(char_type c)
 {
 	return c >= 0x064b && c <= 0x0652;
 }
 
 
-bool Encodings::is_arabic(char_type c)
+bool Encodings::isArabicChar(char_type c)
 {
-	return c >= arabic_start && c <= arabic_end &&
-	       arabic_table[c-arabic_start][0];
+	return c >= arabic_start && c <= arabic_end
+		&& arabic_table[c-arabic_start][0];
 }
 
 
-char_type Encodings::transformChar(char_type c,
-				      Encodings::Letter_Form form)
+char_type Encodings::transformChar(char_type c, Encodings::LetterForm form)
 {
-	if (!is_arabic(c))
-		return c;
-
-	return arabic_table[c-arabic_start][form];
+	return isArabicChar(c) ? arabic_table[c-arabic_start][form] : c;
 }
 
 
@@ -422,15 +622,14 @@ bool Encodings::isKnownScriptChar(char_type const c, string & preamble)
 	if (it == unicodesymbols.end())
 		return false;
 
-	if (it->second.preamble != "textgreek" &&
-	    it->second.preamble != "textcyr")
+	if (it->second.textpreamble != "textgreek" && it->second.textpreamble != "textcyr")
 		return false;
 
 	if (preamble.empty()) {
-		preamble = it->second.preamble;
+		preamble = it->second.textpreamble;
 		return true;
 	}
-	return it->second.preamble == preamble;
+	return it->second.textpreamble == preamble;
 }
 
 
@@ -440,19 +639,22 @@ bool Encodings::isForced(char_type c)
 }
 
 
-Encoding const * Encodings::getFromLyXName(string const & name) const
+bool Encodings::isMathAlpha(char_type c)
 {
-	EncodingList::const_iterator it = encodinglist.find(name);
-	if (it != encodinglist.end())
-		return &it->second;
-	else
-		return 0;
+	return mathalpha.count(c);
 }
 
 
-Encoding const * Encodings::getFromLaTeXName(string const & name) const
+Encoding const * Encodings::fromLyXName(string const & name) const
 {
-	// We don't use std::find_if because it makes copies of the pairs in
+	EncodingList::const_iterator const it = encodinglist.find(name);
+	return it != encodinglist.end() ? &it->second : 0;
+}
+
+
+Encoding const * Encodings::fromLaTeXName(string const & name) const
+{
+	// We don't use find_if because it makes copies of the pairs in
 	// the map.
 	// This linear search is OK since we don't have many encodings.
 	// Users could even optimize it by putting the encodings they use
@@ -474,79 +676,116 @@ void Encodings::read(FileName const & encfile, FileName const & symbolsfile)
 {
 	// We must read the symbolsfile first, because the Encoding
 	// constructor depends on it.
-	Lexer symbolslex(0, 0);
+	Lexer symbolslex;
 	symbolslex.setFile(symbolsfile);
+	bool getNextToken = true;
 	while (symbolslex.isOK()) {
 		char_type symbol;
 		CharInfo info;
 		string flags;
 
-		if (symbolslex.next(true)) {
-			std::istringstream is(symbolslex.getString());
-			// reading symbol directly does not work if
-			// char_type == std::wchar_t.
-			boost::uint32_t tmp;
-			if(!(is >> std::hex >> tmp))
+		if (getNextToken) {
+			if (!symbolslex.next(true))
 				break;
-			symbol = tmp;
 		} else
+			getNextToken = true;
+
+		istringstream is(symbolslex.getString());
+		// reading symbol directly does not work if
+		// char_type == wchar_t.
+		boost::uint32_t tmp;
+		if(!(is >> hex >> tmp))
 			break;
-		if (symbolslex.next(true))
-			info.command = symbolslex.getDocString();
-		else
+		symbol = tmp;
+
+		if (!symbolslex.next(true))
 			break;
-		if (symbolslex.next(true))
-			info.preamble = symbolslex.getString();
-		else
+		info.textcommand = symbolslex.getDocString();
+		if (!symbolslex.next(true))
 			break;
-		if (symbolslex.next(true))
-			flags = symbolslex.getString();
-		else
+		info.textpreamble = symbolslex.getString();
+		if (!symbolslex.next(true))
 			break;
+		flags = symbolslex.getString();
 
 		info.combining = false;
-		info.feature = false;
+		info.textfeature = false;
 		info.force = false;
 		while (!flags.empty()) {
 			string flag;
-			flags = support::split(flags, flag, ',');
-			if (flag == "combining")
+			flags = split(flags, flag, ',');
+			if (flag == "combining") {
 				info.combining = true;
-			else if (flag == "force") {
+			} else if (flag == "force") {
 				info.force = true;
 				forced.insert(symbol);
-			} else
+			} else if (flag == "mathalpha") {
+				mathalpha.insert(symbol);
+			} else {
 				lyxerr << "Ignoring unknown flag `" << flag
 				       << "' for symbol `0x"
-				       << std::hex << symbol << std::dec
+				       << hex << symbol << dec
 				       << "'." << endl;
+			}
+		}
+		// mathcommand and mathpreamble have been added for 1.6.0.
+		// make them optional so that old files still work.
+		int const lineno = symbolslex.lineNumber();
+		bool breakout = false;
+		if (symbolslex.next(true)) {
+			if (symbolslex.lineNumber() != lineno) {
+				// line in old format without mathcommand and mathpreamble
+				getNextToken = false;
+			} else {
+				info.mathcommand = symbolslex.getDocString();
+				if (symbolslex.next(true)) {
+					if (symbolslex.lineNumber() != lineno) {
+						// line in new format with mathcommand only
+						getNextToken = false;
+					} else {
+						// line in new format with mathcommand and mathpreamble
+						info.mathpreamble = symbolslex.getString();
+					}
+				} else
+					breakout = true;
+			}
+		} else {
+			breakout = true;
 		}
 
-		if (!info.preamble.empty())
-			info.feature = info.preamble[0] != '\\';
+		if (!info.textpreamble.empty())
+			info.textfeature = info.textpreamble[0] != '\\';
+		if (!info.mathpreamble.empty())
+			info.mathfeature = info.mathpreamble[0] != '\\';
 
-		LYXERR(Debug::INFO)
-			<< "Read unicode symbol " << symbol << " '"
-			<< to_utf8(info.command) << "' '" << info.preamble
-			<< "' " << info.combining << ' ' << info.feature
-			<< endl;
-		unicodesymbols[symbol] = info;
+		LYXERR(Debug::INFO, "Read unicode symbol " << symbol << " '"
+			<< to_utf8(info.textcommand) << "' '" << info.textpreamble
+			<< "' " << info.combining << ' ' << info.textfeature
+			<< " '" << to_utf8(info.mathcommand) << "' '"
+			<< info.mathpreamble << "' " << info.mathfeature);
+
+		// we assume that at least one command is nonempty when using unicodesymbols
+		if (!info.textcommand.empty() || !info.mathcommand.empty())
+			unicodesymbols[symbol] = info;
+
+		if (breakout)
+			break;
 	}
 
 	// Now read the encodings
-	enum Encodingtags {
+	enum {
 		et_encoding = 1,
 		et_end,
-		et_last
 	};
 
-	struct keyword_item encodingtags[et_last - 1] = {
+	LexerKeyword encodingtags[] = {
 		{ "encoding", et_encoding },
 		{ "end", et_end }
 	};
 
-	Lexer lex(encodingtags, et_last - 1);
+	Lexer lex(encodingtags);
 	lex.setFile(encfile);
+	lex.setContext("Encodings::read");
 	while (lex.isOK()) {
 		switch (lex.lex()) {
 		case et_encoding:
@@ -556,17 +795,19 @@ void Encodings::read(FileName const & encfile, FileName const & symbolsfile)
 			lex.next();
 			string const latexname = lex.getString();
 			lex.next();
+			string const guiname = lex.getString();
+			lex.next();
 			string const iconvname = lex.getString();
 			lex.next();
 			string const width = lex.getString();
-			bool fixedwidth = true;
+			bool fixedwidth = false;
 			if (width == "fixed")
 				fixedwidth = true;
 			else if (width == "variable")
 				fixedwidth = false;
 			else
-				lex.printError("Encodings::read: "
-					       "Unknown width: `$$Token'");
+				lex.printError("Unknown width");
+
 			lex.next();
 			string const p = lex.getString();
 			Encoding::Package package = Encoding::none;
@@ -576,26 +817,26 @@ void Encodings::read(FileName const & encfile, FileName const & symbolsfile)
 				package = Encoding::inputenc;
 			else if (p == "CJK")
 				package = Encoding::CJK;
+			else if (p == "japanese")
+				package = Encoding::japanese;
 			else
-				lex.printError("Encodings::read: "
-					       "Unknown package: `$$Token'");
-			LYXERR(Debug::INFO) << "Reading encoding " << name << endl;
+				lex.printError("Unknown package");
+
+			LYXERR(Debug::INFO, "Reading encoding " << name);
 			encodinglist[name] = Encoding(name, latexname,
-						      iconvname, fixedwidth,
-						      package);
+				guiname, iconvname, fixedwidth, package);
+
 			if (lex.lex() != et_end)
-				lex.printError("Encodings::read: "
-					       "missing end");
+				lex.printError("Missing end");
 			break;
 		}
 		case et_end:
-			lex.printError("Encodings::read: Misplaced end");
+			lex.printError("Misplaced end");
 			break;
 		case Lexer::LEX_FEOF:
 			break;
 		default:
-			lex.printError("Encodings::read: "
-				       "Unknown tag: `$$Token'");
+			lex.printError("Unknown tag");
 			break;
 		}
 	}
