@@ -17,42 +17,35 @@
 #include "Font.h"
 
 #include "BufferParams.h" // stateText
-#include "debug.h"
-#include "Encoding.h"
-#include "gettext.h"
-#include "Language.h"
 #include "Color.h"
+#include "Encoding.h"
+#include "Language.h"
+#include "LaTeXFeatures.h"
 #include "Lexer.h"
 #include "LyXRC.h"
 #include "output_latex.h"
 #include "OutputParams.h"
 
+#include "support/lassert.h"
+#include "support/convert.h"
+#include "support/debug.h"
+#include "support/gettext.h"
 #include "support/lstrings.h"
 
-using std::endl;
-using std::string;
-using std::ostream;
-using std::pair;
-
-#ifndef CXX_GLOBAL_CSTD
-using std::strlen;
-#endif
+#include <cstring>
 
+using namespace std;
+using namespace lyx::support;
 
 namespace lyx {
 
-using support::ascii_lowercase;
-using support::bformat;
-using support::rtrim;
-using support::subst;
-
 //
 // Names for the GUI
 //
 
 namespace {
 
-char const * GUIFamilyNames[Font::NUM_FAMILIES + 2 /* default & error */] =
+char const * GUIFamilyNames[NUM_FAMILIES + 2 /* default & error */] =
 { N_("Roman"), N_("Sans Serif"), N_("Typewriter"), N_("Symbol"),
   "cmr", "cmsy", "cmm", "cmex", "msa", "msb", "eufrak", "wasy", "esint",
   N_("Inherit"), N_("Ignore") };
@@ -76,7 +69,7 @@ char const * GUIMiscNames[5] =
 //
 // Strings used to read and write .lyx format files
 //
-char const * LyXFamilyNames[Font::NUM_FAMILIES + 2 /* default & error */] =
+char const * LyXFamilyNames[NUM_FAMILIES + 2 /* default & error */] =
 { "roman", "sans", "typewriter", "symbol",
   "cmr", "cmsy", "cmm", "cmex", "msa", "msb", "eufrak", "wasy", "esint",
   "default", "error" };
@@ -114,253 +107,30 @@ char const * LaTeXSizeNames[14] =
 } // namespace anon
 
 
-// Initialize static member
-Font::FontBits Font::sane = {
-	ROMAN_FAMILY,
-	MEDIUM_SERIES,
-	UP_SHAPE,
-	SIZE_NORMAL,
-	Color::none,
-	OFF,
-	OFF,
-	OFF,
-	OFF };
-
-// Initialize static member
-Font::FontBits Font::inherit = {
-	INHERIT_FAMILY,
-	INHERIT_SERIES,
-	INHERIT_SHAPE,
-	INHERIT_SIZE,
-	Color::inherit,
-	INHERIT,
-	INHERIT,
-	INHERIT,
-	OFF };
-
-// Initialize static member
-Font::FontBits Font::ignore = {
-	IGNORE_FAMILY,
-	IGNORE_SERIES,
-	IGNORE_SHAPE,
-	IGNORE_SIZE,
-	Color::ignore,
-	IGNORE,
-	IGNORE,
-	IGNORE,
-	IGNORE };
-
-
-bool operator==(Font::FontBits const & lhs,
-		Font::FontBits const & rhs)
+Font::Font(FontInfo bits, Language const * l)
+	: bits_(bits), lang_(l), open_encoding_(false)
 {
-	return lhs.family == rhs.family &&
-		lhs.series == rhs.series &&
-		lhs.shape == rhs.shape &&
-		lhs.size == rhs.size &&
-		lhs.color == rhs.color &&
-		lhs.emph == rhs.emph &&
-		lhs.underbar == rhs.underbar &&
-		lhs.noun == rhs.noun &&
-		lhs.number == rhs.number;
-}
-
-
-Font::Font()
-	: bits(sane), lang(default_language), open_encoding_(false)
-{}
-
-
-Font::Font(Font::FONT_INIT1)
-	: bits(inherit), lang(default_language), open_encoding_(false)
-{}
-
-
-Font::Font(Font::FONT_INIT2)
-	: bits(ignore), lang(ignore_language), open_encoding_(false)
-{}
-
-
-Font::Font(Font::FONT_INIT3)
-	: bits(sane), lang(default_language), open_encoding_(false)
-{}
-
-
-Font::Font(Font::FONT_INIT1, Language const * l)
-	: bits(inherit), lang(l), open_encoding_(false)
-{}
-
-
-Font::Font(Font::FONT_INIT2, Language const * l)
-	: bits(ignore), lang(l), open_encoding_(false)
-{}
-
-
-Font::Font(Font::FONT_INIT3, Language const * l)
-	: bits(sane), lang(l), open_encoding_(false)
-{}
-
-
-
-Color_color Font::color() const
-{
-	return Color::color(bits.color);
+	if (!lang_)
+		lang_ = default_language;
 }
 
 
 bool Font::isRightToLeft() const
 {
-	return lang->rightToLeft();
+	return lang_->rightToLeft();
 }
 
 
 bool Font::isVisibleRightToLeft() const
 {
-	return (lang->rightToLeft() &&
-		number() != ON);
-}
-
-
-void Font::setFamily(Font::FONT_FAMILY f)
-{
-	bits.family = f;
-}
-
-
-void Font::setSeries(Font::FONT_SERIES s)
-{
-	bits.series = s;
-}
-
-
-void Font::setShape(Font::FONT_SHAPE s)
-{
-	bits.shape = s;
-}
-
-
-void Font::setSize(Font::FONT_SIZE s)
-{
-	bits.size = s;
-}
-
-
-void Font::setEmph(Font::FONT_MISC_STATE e)
-{
-	bits.emph = e;
-}
-
-
-void Font::setUnderbar(Font::FONT_MISC_STATE u)
-{
-	bits.underbar = u;
-}
-
-
-void Font::setNoun(Font::FONT_MISC_STATE n)
-{
-	bits.noun = n;
-}
-
-
-void Font::setColor(Color_color c)
-{
-	bits.color = int(c);
+	return (lang_->rightToLeft() &&
+		bits_.number() != FONT_ON);
 }
 
 
 void Font::setLanguage(Language const * l)
 {
-	lang = l;
-}
-
-
-void Font::setNumber(Font::FONT_MISC_STATE n)
-{
-	bits.number = n;
-}
-
-
-/// Decreases font size by one
-Font & Font::decSize()
-{
-	switch (size()) {
-	case SIZE_HUGER:        setSize(SIZE_HUGE);     break;
-	case SIZE_HUGE:         setSize(SIZE_LARGEST);  break;
-	case SIZE_LARGEST:      setSize(SIZE_LARGER);   break;
-	case SIZE_LARGER:       setSize(SIZE_LARGE);    break;
-	case SIZE_LARGE:        setSize(SIZE_NORMAL);   break;
-	case SIZE_NORMAL:       setSize(SIZE_SMALL);    break;
-	case SIZE_SMALL:        setSize(SIZE_FOOTNOTE); break;
-	case SIZE_FOOTNOTE:     setSize(SIZE_SCRIPT);   break;
-	case SIZE_SCRIPT:       setSize(SIZE_TINY);     break;
-	case SIZE_TINY:         break;
-	case INCREASE_SIZE:
-		lyxerr << "Can't Font::decSize on INCREASE_SIZE" << endl;
-		break;
-	case DECREASE_SIZE:
-		lyxerr <<"Can't Font::decSize on DECREASE_SIZE" << endl;
-		break;
-	case INHERIT_SIZE:
-		lyxerr <<"Can't Font::decSize on INHERIT_SIZE" << endl;
-		break;
-	case IGNORE_SIZE:
-		lyxerr <<"Can't Font::decSize on IGNORE_SIZE" << endl;
-		break;
-	}
-	return *this;
-}
-
-
-/// Increases font size by one
-Font & Font::incSize()
-{
-	switch (size()) {
-	case SIZE_HUGER:	break;
-	case SIZE_HUGE:         setSize(SIZE_HUGER);    break;
-	case SIZE_LARGEST:      setSize(SIZE_HUGE);     break;
-	case SIZE_LARGER:       setSize(SIZE_LARGEST);  break;
-	case SIZE_LARGE:        setSize(SIZE_LARGER);   break;
-	case SIZE_NORMAL:       setSize(SIZE_LARGE);    break;
-	case SIZE_SMALL:        setSize(SIZE_NORMAL);   break;
-	case SIZE_FOOTNOTE:     setSize(SIZE_SMALL);    break;
-	case SIZE_SCRIPT:       setSize(SIZE_FOOTNOTE); break;
-	case SIZE_TINY:         setSize(SIZE_SCRIPT);   break;
-	case INCREASE_SIZE:
-		lyxerr <<"Can't Font::incSize on INCREASE_SIZE" << endl;
-		break;
-	case DECREASE_SIZE:
-		lyxerr <<"Can't Font::incSize on DECREASE_SIZE" << endl;
-		break;
-	case INHERIT_SIZE:
-		lyxerr <<"Can't Font::incSize on INHERIT_SIZE" << endl;
-		break;
-	case IGNORE_SIZE:
-		lyxerr <<"Can't Font::incSize on IGNORE_SIZE" << endl;
-		break;
-	}
-	return *this;
-}
-
-
-/// Updates a misc setting according to request
-Font::FONT_MISC_STATE Font::setMisc(FONT_MISC_STATE newfont,
-					  FONT_MISC_STATE org)
-{
-	if (newfont == TOGGLE) {
-		if (org == ON)
-			return OFF;
-		else if (org == OFF)
-			return ON;
-		else {
-			lyxerr <<"Font::setMisc: Need state"
-				" ON or OFF to toggle. Setting to ON" << endl;
-			return ON;
-		}
-	} else if (newfont == IGNORE)
-		return org;
-	else
-		return newfont;
+	lang_ = l;
 }
 
 
@@ -369,169 +139,68 @@ void Font::update(Font const & newfont,
 		     Language const * document_language,
 		     bool toggleall)
 {
-	if (newfont.family() == family() && toggleall)
-		setFamily(INHERIT_FAMILY); // toggle 'back'
-	else if (newfont.family() != IGNORE_FAMILY)
-		setFamily(newfont.family());
-	// else it's IGNORE_SHAPE
+	bits_.update(newfont.fontInfo(), toggleall);
 
-	// "Old" behaviour: "Setting" bold will toggle bold on/off.
-	switch (newfont.series()) {
-	case BOLD_SERIES:
-		// We toggle...
-		if (series() == BOLD_SERIES && toggleall)
-			setSeries(MEDIUM_SERIES);
-		else
-			setSeries(BOLD_SERIES);
-		break;
-	case MEDIUM_SERIES:
-	case INHERIT_SERIES:
-		setSeries(newfont.series());
-		break;
-	case IGNORE_SERIES:
-		break;
-	}
-
-	if (newfont.shape() == shape() && toggleall)
-		setShape(INHERIT_SHAPE); // toggle 'back'
-	else if (newfont.shape() != IGNORE_SHAPE)
-		setShape(newfont.shape());
-	// else it's IGNORE_SHAPE
-
-	if (newfont.size() != IGNORE_SIZE) {
-		if (newfont.size() == INCREASE_SIZE)
-			incSize();
-		else if (newfont.size() == DECREASE_SIZE)
-			decSize();
-		else
-			setSize(newfont.size());
-	}
-
-	setEmph(setMisc(newfont.emph(), emph()));
-	setUnderbar(setMisc(newfont.underbar(), underbar()));
-	setNoun(setMisc(newfont.noun(), noun()));
-
-	setNumber(setMisc(newfont.number(), number()));
 	if (newfont.language() == language() && toggleall)
 		if (language() == document_language)
 			setLanguage(default_language);
 		else
 			setLanguage(document_language);
+	else if (newfont.language() == reset_language)
+		setLanguage(document_language);
 	else if (newfont.language() != ignore_language)
 		setLanguage(newfont.language());
-
-	if (newfont.color() == color() && toggleall)
-		setColor(Color::inherit); // toggle 'back'
-	else if (newfont.color() != Color::ignore)
-		setColor(newfont.color());
 }
 
 
-/// Reduce font to fall back to template where possible
-void Font::reduce(Font const & tmplt)
+docstring const stateText(FontInfo const & f)
 {
-	if (family() == tmplt.family())
-		setFamily(INHERIT_FAMILY);
-	if (series() == tmplt.series())
-		setSeries(INHERIT_SERIES);
-	if (shape() == tmplt.shape())
-		setShape(INHERIT_SHAPE);
-	if (size() == tmplt.size())
-		setSize(INHERIT_SIZE);
-	if (emph() == tmplt.emph())
-		setEmph(INHERIT);
-	if (underbar() == tmplt.underbar())
-		setUnderbar(INHERIT);
-	if (noun() == tmplt.noun())
-		setNoun(INHERIT);
-	if (color() == tmplt.color())
-		setColor(Color::inherit);
-}
+	odocstringstream os;
+	if (f.family() != INHERIT_FAMILY)
+		os << _(GUIFamilyNames[f.family()]) << ", ";
+	if (f.series() != INHERIT_SERIES)
+		os << _(GUISeriesNames[f.series()]) << ", ";
+	if (f.shape() != INHERIT_SHAPE)
+		os << _(GUIShapeNames[f.shape()]) << ", ";
+	if (f.size() != FONT_SIZE_INHERIT)
+		os << _(GUISizeNames[f.size()]) << ", ";
+	if (f.color() != Color_inherit)
+		os << lcolor.getGUIName(f.color()) << ", ";
+	// FIXME: uncomment this when we support background.
+	//if (f.background() != Color_inherit)
+	//	os << lcolor.getGUIName(f.background()) << ", ";
+	if (f.emph() != FONT_INHERIT)
+		os << bformat(_("Emphasis %1$s, "),
+			      _(GUIMiscNames[f.emph()]));
+	if (f.underbar() != FONT_INHERIT)
+		os << bformat(_("Underline %1$s, "),
+			      _(GUIMiscNames[f.underbar()]));
+	if (f.noun() != FONT_INHERIT)
+		os << bformat(_("Noun %1$s, "),
+			      _(GUIMiscNames[f.noun()]));
+	if (f == inherit_font)
+		os << _("Default") << ", ";
 
-
-/// Realize font from a template
-Font & Font::realize(Font const & tmplt)
-{
-	if (bits == inherit) {
-		bits = tmplt.bits;
-		return *this;
-	}
-
-	if (bits.family == INHERIT_FAMILY)
-		bits.family = tmplt.bits.family;
-
-	if (bits.series == INHERIT_SERIES)
-		bits.series = tmplt.bits.series;
-
-	if (bits.shape == INHERIT_SHAPE)
-		bits.shape = tmplt.bits.shape;
-
-	if (bits.size == INHERIT_SIZE)
-		bits.size = tmplt.bits.size;
-
-	if (bits.emph == INHERIT)
-		bits.emph = tmplt.bits.emph;
-
-	if (bits.underbar == INHERIT)
-		bits.underbar = tmplt.bits.underbar;
-
-	if (bits.noun == INHERIT)
-		bits.noun = tmplt.bits.noun;
-
-	if (bits.color == Color::inherit)
-		bits.color = tmplt.bits.color;
-
-	return *this;
-}
-
-
-/// Is font resolved?
-bool Font::resolved() const
-{
-	return (family() != INHERIT_FAMILY && series() != INHERIT_SERIES &&
-		shape() != INHERIT_SHAPE && size() != INHERIT_SIZE &&
-		emph() != INHERIT && underbar() != INHERIT &&
-		noun() != INHERIT &&
-		color() != Color::inherit);
+	return os.str();
 }
 
 
 docstring const Font::stateText(BufferParams * params) const
 {
 	odocstringstream os;
-	if (family() != INHERIT_FAMILY)
-		os << _(GUIFamilyNames[family()]) << ", ";
-	if (series() != INHERIT_SERIES)
-		os << _(GUISeriesNames[series()]) << ", ";
-	if (shape() != INHERIT_SHAPE)
-		os << _(GUIShapeNames[shape()]) << ", ";
-	if (size() != INHERIT_SIZE)
-		os << _(GUISizeNames[size()]) << ", ";
-	if (color() != Color::inherit)
-		os << lcolor.getGUIName(color()) << ", ";
-	if (emph() != INHERIT)
-		os << bformat(_("Emphasis %1$s, "),
-			      _(GUIMiscNames[emph()]));
-	if (underbar() != INHERIT)
-		os << bformat(_("Underline %1$s, "),
-			      _(GUIMiscNames[underbar()]));
-	if (noun() != INHERIT)
-		os << bformat(_("Noun %1$s, "),
-			      _(GUIMiscNames[noun()]));
-	if (bits == inherit)
-		os << _("Default") << ", ";
+	os << lyx::stateText(bits_);
 	if (!params || (language() != params->language))
 		os << bformat(_("Language: %1$s, "),
 			      _(language()->display()));
-	if (number() != OFF)
+	if (bits_.number() != FONT_OFF)
 		os << bformat(_("  Number %1$s"),
-			      _(GUIMiscNames[number()]));
+			      _(GUIMiscNames[bits_.number()]));
 	return rtrim(os.str(), ", ");
 }
 
 
 // Set family according to lyx format string
-Font & Font::setLyXFamily(string const & fam)
+void setLyXFamily(string const & fam, FontInfo & f)
 {
 	string const s = ascii_lowercase(fam);
 
@@ -540,16 +209,15 @@ Font & Font::setLyXFamily(string const & fam)
 	       LyXFamilyNames[i] != string("error"))
 		++i;
 	if (s == LyXFamilyNames[i])
-		setFamily(Font::FONT_FAMILY(i));
+		f.setFamily(FontFamily(i));
 	else
-		lyxerr << "Font::setLyXFamily: Unknown family `"
+		lyxerr << "setLyXFamily: Unknown family `"
 		       << s << '\'' << endl;
-	return *this;
 }
 
 
 // Set series according to lyx format string
-Font & Font::setLyXSeries(string const & ser)
+void setLyXSeries(string const & ser, FontInfo & f)
 {
 	string const s = ascii_lowercase(ser);
 
@@ -557,16 +225,15 @@ Font & Font::setLyXSeries(string const & ser)
 	while (LyXSeriesNames[i] != s &&
 	       LyXSeriesNames[i] != string("error")) ++i;
 	if (s == LyXSeriesNames[i]) {
-		setSeries(Font::FONT_SERIES(i));
+		f.setSeries(FontSeries(i));
 	} else
-		lyxerr << "Font::setLyXSeries: Unknown series `"
+		lyxerr << "setLyXSeries: Unknown series `"
 		       << s << '\'' << endl;
-	return *this;
 }
 
 
 // Set shape according to lyx format string
-Font & Font::setLyXShape(string const & sha)
+void setLyXShape(string const & sha, FontInfo & f)
 {
 	string const s = ascii_lowercase(sha);
 
@@ -574,64 +241,62 @@ Font & Font::setLyXShape(string const & sha)
 	while (LyXShapeNames[i] != s && LyXShapeNames[i] != string("error"))
 			++i;
 	if (s == LyXShapeNames[i])
-		setShape(Font::FONT_SHAPE(i));
+		f.setShape(FontShape(i));
 	else
 		lyxerr << "Font::setLyXShape: Unknown shape `"
 		       << s << '\'' << endl;
-	return *this;
 }
 
 
 // Set size according to lyx format string
-Font & Font::setLyXSize(string const & siz)
+void setLyXSize(string const & siz, FontInfo & f)
 {
 	string const s = ascii_lowercase(siz);
 	int i = 0;
 	while (LyXSizeNames[i] != s && LyXSizeNames[i] != string("error"))
 		++i;
 	if (s == LyXSizeNames[i]) {
-		setSize(Font::FONT_SIZE(i));
+		f.setSize(FontSize(i));
 	} else
 		lyxerr << "Font::setLyXSize: Unknown size `"
 		       << s << '\'' << endl;
-	return *this;
 }
 
 
 // Set size according to lyx format string
-Font::FONT_MISC_STATE Font::setLyXMisc(string const & siz)
+FontState Font::setLyXMisc(string const & siz)
 {
 	string const s = ascii_lowercase(siz);
 	int i = 0;
 	while (LyXMiscNames[i] != s &&
 	       LyXMiscNames[i] != string("error")) ++i;
 	if (s == LyXMiscNames[i])
-		return FONT_MISC_STATE(i);
+		return FontState(i);
 	lyxerr << "Font::setLyXMisc: Unknown misc flag `"
 	       << s << '\'' << endl;
-	return OFF;
+	return FONT_OFF;
 }
 
 
 /// Sets color after LyX text format
-Font & Font::setLyXColor(string const & col)
+void setLyXColor(string const & col, FontInfo & f)
 {
-	setColor(lcolor.getFromLyXName(col));
-	return *this;
+	f.setColor(lcolor.getFromLyXName(col));
 }
 
 
 // Returns size in latex format
 string const Font::latexSize() const
 {
-	return LaTeXSizeNames[size()];
+	return LaTeXSizeNames[bits_.size()];
 }
 
 
 // Read a font definition from given file in lyx format
 // Used for layouts
-Font & Font::lyxRead(Lexer & lex)
+FontInfo lyxRead(Lexer & lex, FontInfo const & fi)
 {
+	FontInfo f = fi;
 	bool error = false;
 	bool finished = false;
 	while (!finished && lex.isOK() && !error) {
@@ -645,48 +310,48 @@ Font & Font::lyxRead(Lexer & lex)
 		} else if (tok == "family") {
 			lex.next();
 			string const ttok = lex.getString();
-			setLyXFamily(ttok);
+			setLyXFamily(ttok, f);
 		} else if (tok == "series") {
 			lex.next();
 			string const ttok = lex.getString();
-			setLyXSeries(ttok);
+			setLyXSeries(ttok, f);
 		} else if (tok == "shape") {
 			lex.next();
 			string const ttok = lex.getString();
-			setLyXShape(ttok);
+			setLyXShape(ttok, f);
 		} else if (tok == "size") {
 			lex.next();
 			string const ttok = lex.getString();
-			setLyXSize(ttok);
+			setLyXSize(ttok, f);
 		} else if (tok == "misc") {
 			lex.next();
 			string const ttok = ascii_lowercase(lex.getString());
 
 			if (ttok == "no_bar") {
-				setUnderbar(OFF);
+				f.setUnderbar(FONT_OFF);
 			} else if (ttok == "no_emph") {
-				setEmph(OFF);
+				f.setEmph(FONT_OFF);
 			} else if (ttok == "no_noun") {
-				setNoun(OFF);
+				f.setNoun(FONT_OFF);
 			} else if (ttok == "emph") {
-				setEmph(ON);
+				f.setEmph(FONT_ON);
 			} else if (ttok == "underbar") {
-				setUnderbar(ON);
+				f.setUnderbar(FONT_ON);
 			} else if (ttok == "noun") {
-				setNoun(ON);
+				f.setNoun(FONT_ON);
 			} else {
-				lex.printError("Illegal misc type `$$Token'");
+				lex.printError("Illegal misc type");
 			}
 		} else if (tok == "color") {
 			lex.next();
 			string const ttok = lex.getString();
-			setLyXColor(ttok);
+			setLyXColor(ttok, f);
 		} else {
-			lex.printError("Unknown tag `$$Token'");
+			lex.printError("Unknown tag");
 			error = true;
 		}
 	}
-	return *this;
+	return f;
 }
 
 
@@ -695,39 +360,42 @@ void Font::lyxWriteChanges(Font const & orgfont,
 			      ostream & os) const
 {
 	os << "\n";
-	if (orgfont.family() != family())
-		os << "\\family " << LyXFamilyNames[family()] << "\n";
-	if (orgfont.series() != series())
-		os << "\\series " << LyXSeriesNames[series()] << "\n";
-	if (orgfont.shape() != shape())
-		os << "\\shape " << LyXShapeNames[shape()] << "\n";
-	if (orgfont.size() != size())
-		os << "\\size " << LyXSizeNames[size()] << "\n";
-	if (orgfont.emph() != emph())
-		os << "\\emph " << LyXMiscNames[emph()] << "\n";
-	if (orgfont.number() != number())
-		os << "\\numeric " << LyXMiscNames[number()] << "\n";
-	if (orgfont.underbar() != underbar()) {
+	if (orgfont.fontInfo().family() != bits_.family())
+		os << "\\family " << LyXFamilyNames[bits_.family()] << "\n";
+	if (orgfont.fontInfo().series() != bits_.series())
+		os << "\\series " << LyXSeriesNames[bits_.series()] << "\n";
+	if (orgfont.fontInfo().shape() != bits_.shape())
+		os << "\\shape " << LyXShapeNames[bits_.shape()] << "\n";
+	if (orgfont.fontInfo().size() != bits_.size())
+		os << "\\size " << LyXSizeNames[bits_.size()] << "\n";
+	if (orgfont.fontInfo().emph() != bits_.emph())
+		os << "\\emph " << LyXMiscNames[bits_.emph()] << "\n";
+	if (orgfont.fontInfo().number() != bits_.number())
+		os << "\\numeric " << LyXMiscNames[bits_.number()] << "\n";
+	if (orgfont.fontInfo().underbar() != bits_.underbar()) {
 		// This is only for backwards compatibility
-		switch (underbar()) {
-		case OFF:	os << "\\bar no\n"; break;
-		case ON:        os << "\\bar under\n"; break;
-		case TOGGLE:	lyxerr << "Font::lyxWriteFontChanges: "
-					"TOGGLE should not appear here!"
+		switch (bits_.underbar()) {
+		case FONT_OFF:	os << "\\bar no\n"; break;
+		case FONT_ON:        os << "\\bar under\n"; break;
+		case FONT_TOGGLE:	lyxerr << "Font::lyxWriteFontChanges: "
+					"FONT_TOGGLE should not appear here!"
 				       << endl;
 		break;
-		case INHERIT:   os << "\\bar default\n"; break;
-		case IGNORE:    lyxerr << "Font::lyxWriteFontChanges: "
+		case FONT_INHERIT:   os << "\\bar default\n"; break;
+		case FONT_IGNORE:    lyxerr << "Font::lyxWriteFontChanges: "
 					"IGNORE should not appear here!"
 				       << endl;
 		break;
 		}
 	}
-	if (orgfont.noun() != noun()) {
-		os << "\\noun " << LyXMiscNames[noun()] << "\n";
+	if (orgfont.fontInfo().noun() != bits_.noun()) {
+		os << "\\noun " << LyXMiscNames[bits_.noun()] << "\n";
 	}
-	if (orgfont.color() != color())
-		os << "\\color " << lcolor.getLyXName(color()) << '\n';
+	if (orgfont.fontInfo().color() != bits_.color())
+		os << "\\color " << lcolor.getLyXName(bits_.color()) << '\n';
+	// FIXME: uncomment this when we support background.
+	//if (orgfont.fontInfo().background() != bits_.background())
+	//	os << "\\color " << lcolor.getLyXName(bits_.background()) << '\n';
 	if (orgfont.language() != language() &&
 	    language() != latex_language) {
 		if (language())
@@ -799,7 +467,7 @@ int Font::latexWriteStartChanges(odocstream & os, BufferParams const & bparams,
 	// the numbers are written Left-to-Right. ArabTeX package
 	// reorders the number automatically but the packages used
 	// for Hebrew and Farsi (Arabi) do not.
-	if (number() == ON && prev.number() != ON
+	if (bits_.number() == FONT_ON && prev.fontInfo().number() != FONT_ON
 		&& (language()->lang() == "hebrew"
 			|| language()->lang() == "farsi" 
 			|| language()->lang() == "arabic_arabi")) {
@@ -807,8 +475,8 @@ int Font::latexWriteStartChanges(odocstream & os, BufferParams const & bparams,
 		count += 9;
 	}
 
-	Font f = *this;
-	f.reduce(base);
+	FontInfo f = bits_;
+	f.reduce(base.bits_);
 
 	if (f.family() != INHERIT_FAMILY) {
 		os << '\\'
@@ -831,30 +499,40 @@ int Font::latexWriteStartChanges(odocstream & os, BufferParams const & bparams,
 		count += strlen(LaTeXShapeNames[f.shape()]) + 2;
 		env = true; //We have opened a new environment
 	}
-	if (f.color() != Color::inherit && f.color() != Color::ignore) {
+	if (f.color() != Color_inherit && f.color() != Color_ignore) {
 		os << "\\textcolor{"
 		   << from_ascii(lcolor.getLaTeXName(f.color()))
 		   << "}{";
 		count += lcolor.getLaTeXName(f.color()).length() + 13;
 		env = true; //We have opened a new environment
 	}
-	if (f.emph() == ON) {
+	// FIXME: uncomment this when we support background.
+	/*
+	if (f.background() != Color_inherit && f.background() != Color_ignore) {
+		os << "\\textcolor{"
+		   << from_ascii(lcolor.getLaTeXName(f.background()))
+		   << "}{";
+		count += lcolor.getLaTeXName(f.background()).length() + 13;
+		env = true; //We have opened a new environment
+	}
+	*/
+	if (f.emph() == FONT_ON) {
 		os << "\\emph{";
 		count += 6;
 		env = true; //We have opened a new environment
 	}
-	if (f.underbar() == ON) {
+	if (f.underbar() == FONT_ON) {
 		os << "\\underbar{";
 		count += 10;
 		env = true; //We have opened a new environment
 	}
 	// \noun{} is a LyX special macro
-	if (f.noun() == ON) {
+	if (f.noun() == FONT_ON) {
 		os << "\\noun{";
 		count += 6;
 		env = true; //We have opened a new environment
 	}
-	if (f.size() != INHERIT_SIZE) {
+	if (f.size() != FONT_SIZE_INHERIT) {
 		// If we didn't open an environment above, we open one here
 		if (!env) {
 			os << '{';
@@ -884,8 +562,8 @@ int Font::latexWriteEndChanges(odocstream & os, BufferParams const & bparams,
 	// reduce the current font to changes against the base
 	// font (of the layout). We use a temporary for this to
 	// avoid changing this font instance, as that would break
-	Font f = *this;
-	f.reduce(base);
+	FontInfo f = bits_;
+	f.reduce(base.bits_);
 
 	if (f.family() != INHERIT_FAMILY) {
 		os << '}';
@@ -902,27 +580,27 @@ int Font::latexWriteEndChanges(odocstream & os, BufferParams const & bparams,
 		++count;
 		env = true; // Size change need not bother about closing env.
 	}
-	if (f.color() != Color::inherit && f.color() != Color::ignore) {
+	if (f.color() != Color_inherit && f.color() != Color_ignore) {
 		os << '}';
 		++count;
 		env = true; // Size change need not bother about closing env.
 	}
-	if (f.emph() == ON) {
+	if (f.emph() == FONT_ON) {
 		os << '}';
 		++count;
 		env = true; // Size change need not bother about closing env.
 	}
-	if (f.underbar() == ON) {
+	if (f.underbar() == FONT_ON) {
 		os << '}';
 		++count;
 		env = true; // Size change need not bother about closing env.
 	}
-	if (f.noun() == ON) {
+	if (f.noun() == FONT_ON) {
 		os << '}';
 		++count;
 		env = true; // Size change need not bother about closing env.
 	}
-	if (f.size() != INHERIT_SIZE) {
+	if (f.size() != FONT_SIZE_INHERIT) {
 		// We only have to close if only size changed
 		if (!env) {
 			os << '}';
@@ -934,7 +612,7 @@ int Font::latexWriteEndChanges(odocstream & os, BufferParams const & bparams,
 	// the numbers are written Left-to-Right. ArabTeX package
 	// reorders the number automatically but the packages used
 	// for Hebrew and Farsi (Arabi) do not.
-	if (number() == ON && next.number() != ON
+	if (bits_.number() == FONT_ON && next.fontInfo().number() != FONT_ON
 		&& (language()->lang() == "hebrew"
 			|| language()->lang() == "farsi"
 			|| language()->lang() == "arabic_arabi")) {
@@ -945,10 +623,10 @@ int Font::latexWriteEndChanges(odocstream & os, BufferParams const & bparams,
 	if (open_encoding_) {
 		// We need to close the encoding even if it does not change
 		// to do correct environment nesting
-		Encoding const * const ascii = encodings.getFromLyXName("ascii");
+		Encoding const * const ascii = encodings.fromLyXName("ascii");
 		pair<bool, int> const c = switchEncoding(os, bparams,
 				runparams, *ascii);
-		BOOST_ASSERT(c.first);
+		LASSERT(c.first, /**/);
 		count += c.second;
 		runparams.encoding = ascii;
 		open_encoding_ = false;
@@ -964,33 +642,164 @@ int Font::latexWriteEndChanges(odocstream & os, BufferParams const & bparams,
 }
 
 
-Color_color Font::realColor() const
+string Font::toString(bool const toggle) const
 {
-	if (color() == Color::none)
-		return Color::foreground;
-	return color();
+	string const lang = (language() == reset_language)
+		? "reset" : language()->lang();
+
+	ostringstream os;
+	os << "family " << bits_.family() << '\n'
+	   << "series " << bits_.series() << '\n'
+	   << "shape " << bits_.shape() << '\n'
+	   << "size " << bits_.size() << '\n'
+	   << "emph " << bits_.emph() << '\n'
+	   << "underbar " << bits_.underbar() << '\n'
+	   << "noun " << bits_.noun() << '\n'
+	   << "number " << bits_.number() << '\n'
+	   << "color " << bits_.color() << '\n'
+	   << "language " << lang << '\n'
+	   << "toggleall " << convert<string>(toggle);
+	return os.str();
 }
 
 
-ostream & operator<<(ostream & os, Font::FONT_MISC_STATE fms)
+bool Font::fromString(string const & data, bool & toggle)
+{
+	istringstream is(data);
+	Lexer lex;
+	lex.setStream(is);
+
+	int nset = 0;
+	while (lex.isOK()) {
+		string token;
+		if (lex.next())
+			token = lex.getString();
+
+		if (token.empty() || !lex.next())
+			break;
+
+		if (token == "family") {
+			int const next = lex.getInteger();
+			bits_.setFamily(FontFamily(next));
+
+		} else if (token == "series") {
+			int const next = lex.getInteger();
+			bits_.setSeries(FontSeries(next));
+
+		} else if (token == "shape") {
+			int const next = lex.getInteger();
+			bits_.setShape(FontShape(next));
+
+		} else if (token == "size") {
+			int const next = lex.getInteger();
+			bits_.setSize(FontSize(next));
+
+		} else if (token == "emph" || token == "underbar" ||
+			   token == "noun" || token == "number") {
+
+			int const next = lex.getInteger();
+			FontState const misc = FontState(next);
+
+			if (token == "emph")
+				bits_.setEmph(misc);
+			else if (token == "underbar")
+				bits_.setUnderbar(misc);
+			else if (token == "noun")
+				bits_.setNoun(misc);
+			else if (token == "number")
+				bits_.setNumber(misc);
+
+		} else if (token == "color") {
+			int const next = lex.getInteger();
+			bits_.setColor(ColorCode(next));
+
+		/**
+		} else if (token == "background") {
+			int const next = lex.getInteger();
+			bits_.setBackground(ColorCode(next));
+		*/
+
+		} else if (token == "language") {
+			string const next = lex.getString();
+			setLanguage(languages.getLanguage(next));
+
+		} else if (token == "toggleall") {
+			toggle = lex.getBool();
+
+		} else {
+			// Unrecognised token
+			break;
+		}
+
+		++nset;
+	}
+	return (nset > 0);
+}
+
+
+void Font::validate(LaTeXFeatures & features) const
+{
+	BufferParams const & bparams = features.bufferParams();
+	Language const * doc_language = bparams.language;
+
+	if (bits_.noun() == FONT_ON) {
+		LYXERR(Debug::LATEX, "font.noun: " << bits_.noun());
+		features.require("noun");
+		LYXERR(Debug::LATEX, "Noun enabled. Font: " << to_utf8(stateText(0)));
+	}
+	switch (bits_.color()) {
+		case Color_none:
+		case Color_inherit:
+		case Color_ignore:
+			// probably we should put here all interface colors used for
+			// font displaying! For now I just add this ones I know of (Jug)
+		case Color_latex:
+		case Color_notelabel:
+			break;
+		default:
+			features.require("color");
+			LYXERR(Debug::LATEX, "Color enabled. Font: " << to_utf8(stateText(0)));
+	}
+
+	// FIXME: Do something for background and soul package?
+
+	if (lang_->babel() != doc_language->babel() &&
+		lang_ != ignore_language &&
+		lang_ != latex_language)
+	{
+		features.useLanguage(lang_);
+		LYXERR(Debug::LATEX, "Found language " << lang_->lang());
+	}
+}
+
+
+ostream & operator<<(ostream & os, FontState fms)
 {
 	return os << int(fms);
 }
 
 
-std::ostream & operator<<(std::ostream & os, Font const & font)
+ostream & operator<<(ostream & os, FontInfo const & f)
 {
 	return os << "font:"
-		<< " family " << font.bits.family
-		<< " series " << font.bits.series
-		<< " shape " << font.bits.shape
-		<< " size " << font.bits.size
-		<< " color " << font.bits.color
-		<< " emph " << font.bits.emph
-		<< " underbar " << font.bits.underbar
-		<< " noun " << font.bits.noun
-		<< " number " << font.bits.number
-		<< " lang: " << (font.lang ? font.lang->lang() : 0);
+		<< " family " << f.family()
+		<< " series " << f.series()
+		<< " shape " << f.shape()
+		<< " size " << f.size()
+		<< " color " << f.color()
+		// FIXME: uncomment this when we support background.
+		//<< " background " << f.background()
+		<< " emph " << f.emph()
+		<< " underbar " << f.underbar()
+		<< " noun " << f.noun()
+		<< " number " << f.number();
+}
+
+
+ostream & operator<<(ostream & os, Font const & font)
+{
+	return os << font.bits_
+		<< " lang: " << (font.lang_ ? font.lang_->lang() : 0);
 }
 
 
