@@ -13,7 +13,6 @@
 #ifndef LATEXFEATURES_H
 #define LATEXFEATURES_H
 
-
 #include "OutputParams.h"
 #include "support/docstring.h"
 
@@ -47,6 +46,8 @@ public:
 	///
 	LaTeXFeatures(Buffer const &, BufferParams const &,
 		      OutputParams const &);
+	/// The color packages
+	std::string const getColorOptions() const;
 	/// The packages needed by the document
 	std::string const getPackages() const;
 	/// The macros definitions needed by the document
@@ -67,8 +68,10 @@ public:
 	void showStruct() const;
 	///
 	void addPreambleSnippet(std::string const &);
-	/// Provide a string name-space to the requirements
+	/// Add a feature name requirements
 	void require(std::string const & name);
+	/// Add a set of feature names requirements
+	void require(std::set<std::string> const & names);
 	/// Which of the required packages are installed?
 	static void getAvailable();
 	/// Is the (required) package available?
@@ -81,7 +84,7 @@ public:
 	*/
 	bool mustProvide(std::string const & name) const;
 	///
-	void useFloat(std::string const & name);
+	void useFloat(std::string const & name, bool subfloat = false);
 	///
 	void useLanguage(Language const *);
 	///
@@ -106,26 +109,28 @@ public:
 private:
 	std::list<docstring> usedLayouts_;
 
+	/// The features that are needed by the document
+	typedef std::set<std::string> Features;
+	///
+	Features features_;
 	/// Static preamble bits from the external material insets
-	typedef std::list<std::string> FeaturesList;
+	typedef std::list<std::string> SnippetList;
 	///
-	FeaturesList features_;
-	///
-	FeaturesList preamble_snippets_;
+	SnippetList preamble_snippets_;
 	/// The available (required) packages
-	typedef std::list<std::string> PackagesList;
+	typedef std::set<std::string> Packages;
 	///
-	static PackagesList packages_;
+	static Packages packages_;
 	///
 	typedef std::set<Language const *> LanguageList;
 	/// used languages (only those that are supported by babel)
 	LanguageList UsedLanguages_;
 	///
-	typedef std::set<std::string> UsedFloats;
+	typedef std::map<std::string, bool> UsedFloats;
 	///
 	UsedFloats usedFloats_;
 	///
-	typedef std::map<docstring , std::string> FileMap;
+	typedef std::map<docstring, std::string> FileMap;
 	///
 	FileMap IncludedFiles_;
 	/** Buffer of the file being processed.
