@@ -1030,10 +1030,13 @@ void LyXFunc::dispatch(FuncRequest const & cmd)
 				theApp()->dispatch(FuncRequest(LFUN_WINDOW_NEW));
 			string const arg = argument;
 			if (arg.empty()) {
-				setErrorMessage(from_ascii(N_("Missing argument")));
+				setErrorMessage(from_utf8(N_("Missing argument")));
 				break;
 			}
-			FileName const fname = i18nLibFileSearch("doc", arg, "lyx");
+			FileName fname = i18nLibFileSearch("doc", arg, "lyx");
+			if (fname.empty()) 
+				fname = i18nLibFileSearch("examples", arg, "lyx");
+
 			if (fname.empty()) {
 				lyxerr << "LyX: unable to find documentation file `"
 							 << arg << "'. Bad installation?" << endl;
@@ -1348,6 +1351,8 @@ void LyXFunc::dispatch(FuncRequest const & cmd)
 		case LFUN_COMMAND_SEQUENCE: {
 			// argument contains ';'-terminated commands
 			string arg = argument;
+			if (theBufferList().isLoaded(buffer))
+				buffer->undo().beginUndoGroup();
 			while (!arg.empty()) {
 				string first;
 				arg = split(arg, first, ';');
@@ -1355,6 +1360,8 @@ void LyXFunc::dispatch(FuncRequest const & cmd)
 				func.origin = cmd.origin;
 				dispatch(func);
 			}
+			if (theBufferList().isLoaded(buffer))
+				buffer->undo().endUndoGroup();
 			break;
 		}
 
@@ -1726,14 +1733,14 @@ void LyXFunc::sendDispatchMessage(docstring const & msg, FuncRequest const & cmd
 
 void LyXFunc::reloadBuffer()
 {
-	FileName filename = lyx_view_->buffer()->fileName();
+	Buffer * buf = lyx_view_->buffer();
+	FileName filename = buf->fileName();
 	// The user has already confirmed that the changes, if any, should
-	// be discarded. So we just release the Buffer and don't call closeBuffer();
-	theBufferList().release(lyx_view_->buffer());
-	Buffer * buf = lyx_view_->loadDocument(filename);
+	// be discarded. So we just reread the file and don't call closeBuffer();
+	bool const success = buf->readFile(filename);
 	docstring const disp_fn = makeDisplayPath(filename.absFilename());
 	docstring str;
-	if (buf) {
+	if (buf && success) {
 		updateLabels(*buf);
 		lyx_view_->setBuffer(buf);
 		buf->errors("Parse");
