@@ -23,39 +23,39 @@ namespace lyx {
  * InsetOptArg. Used to insert a short version of sectioning header etc.
  * automatically, or other optional LaTeX arguments
  */
-class InsetOptArg : public InsetCollapsable {
+class InsetOptArg : public InsetCollapsable
+{
 public:
-	InsetOptArg(BufferParams const &);
-
-	/// code of the inset
-	Inset::Code lyxCode() const { return Inset::OPTARG_CODE; }
-	/// return an message upon editing
-	virtual docstring const editMessage() const;
-
-	/// Standard LaTeX output -- short-circuited
-	int latex(Buffer const &, odocstream &,
-		  OutputParams const &) const;
-	/// Standard plain text output -- short-circuited
-	int plaintext(Buffer const &, odocstream &,
-		      OutputParams const &) const;
-	/// Standard DocBook output -- short-circuited
-	int docbook(Buffer const &, odocstream &,
-		    OutputParams const &) const;
+	///
+	InsetOptArg(Buffer const &);
 
 	/// Outputting the optional parameter of a LaTeX command
-	int latexOptional(Buffer const &, odocstream &,
-			  OutputParams const &) const;
+	int latexOptional(odocstream &, OutputParams const &) const;
+
+private:
+	/// code of the inset
+	InsetCode lyxCode() const { return OPTARG_CODE; }
+	///
+	docstring name() const { return from_ascii("OptArg"); }
+	/// return an message upon editing
+	docstring editMessage() const;
+
+	/// Standard LaTeX output -- short-circuited
+	int latex(odocstream &, OutputParams const &) const;
+	/// Standard plain text output -- short-circuited
+	int plaintext(odocstream &, OutputParams const &) const;
+	/// Standard DocBook output -- short-circuited
+	int docbook(odocstream &, OutputParams const &) const;
+	///
+	virtual bool forcePlainLayout(idx_type = 0) const { return true; }
+
 	/// Write out to the .lyx file
-	void write(Buffer const & buf, std::ostream & os) const;
+	void write(std::ostream & os) const;
 
 	/// should paragraph indendation be ommitted in any case?
-	virtual bool neverIndent(Buffer const &) const { return true; }
-protected:
-	InsetOptArg(InsetOptArg const &);
+	bool neverIndent() const { return true; }
 	///
-	bool getStatus(Cursor & cur, FuncRequest const & cmd, FuncStatus &) const;
-private:
-	virtual std::auto_ptr<Inset> doClone() const;
+	Inset * clone() const { return new InsetOptArg(*this); }
 };
 
 
