@@ -4,7 +4,6 @@
  * This file is part of LyX, the document processor.
  * Licence details can be found in the file COPYING.
  *
- * \author Jrgen Vigna
  * \author Lars Gullik Bjnnes
  * \author Matthias Ettrich
  * \author Andr Pnitz
@@ -33,14 +32,13 @@
 
 // Lgb
 
-#ifndef INSETTABULAR_H
-#define INSETTABULAR_H
+#ifndef INSET_TABULAR_H
+#define INSET_TABULAR_H
 
 #include "Inset.h"
-#include "MailInset.h"
-#include "Length.h"
 #include "InsetText.h"
-
+#include "Layout.h"
+#include "Length.h"
 
 #include <boost/shared_ptr.hpp>
 
@@ -49,13 +47,15 @@
 
 namespace lyx {
 
-class FuncStatus;
-class Lexer;
-class BufferView;
 class Buffer;
 class BufferParams;
-class Paragraph;
+class BufferView;
+class CompletionList;
 class CursorSlice;
+class InsetTableCell;
+class FuncStatus;
+class Lexer;
+class Paragraph;
 
 namespace frontend { class Painter; }
 
@@ -67,7 +67,7 @@ class OutputParams;
 //
 // A helper struct for tables
 //
-class Tabular  {
+class Tabular {
 public:
 	///
 	enum Feature {
@@ -106,14 +106,6 @@ public:
 		///
 		VALIGN_MIDDLE,
 		///
-		M_TOGGLE_LINE_TOP,
-		///
-		M_TOGGLE_LINE_BOTTOM,
-		///
-		M_TOGGLE_LINE_LEFT,
-		///
-		M_TOGGLE_LINE_RIGHT,
-		///
 		M_ALIGN_LEFT,
 		///
 		M_ALIGN_RIGHT,
@@ -168,6 +160,8 @@ public:
 		///
 		SET_LTNEWPAGE,
 		///
+		TOGGLE_LTCAPTION,
+		///
 		SET_SPECIAL_COLUMN,
 		///
 		SET_SPECIAL_MULTI,
@@ -182,6 +176,8 @@ public:
 		///
 		SET_INTERLINE_SPACE,
 		///
+		SET_BORDER_LINES,
+		///
 		LAST_ACTION
 	};
 	///
@@ -239,17 +235,16 @@ public:
 	static const idx_type npos = static_cast<idx_type>(-1);
 
 	/// constructor
-	Tabular(BufferParams const &, col_type columns_arg,
-		   row_type rows_arg);
+	Tabular(Buffer &, col_type columns_arg, row_type rows_arg);
 
 	/// Returns true if there is a topline, returns false if not
-	bool topLine(idx_type cell, bool wholerow = false) const;
+	bool topLine(idx_type cell) const;
 	/// Returns true if there is a topline, returns false if not
-	bool bottomLine(idx_type cell, bool wholerow = false) const;
+	bool bottomLine(idx_type cell) const;
 	/// Returns true if there is a topline, returns false if not
-	bool leftLine(idx_type cell, bool wholecolumn = false) const;
+	bool leftLine(idx_type cell) const;
 	/// Returns true if there is a topline, returns false if not
-	bool rightLine(idx_type cell, bool wholecolumn = false) const;
+	bool rightLine(idx_type cell) const;
 
 	///
 	bool topAlreadyDrawn(idx_type cell) const;
@@ -266,32 +261,40 @@ public:
 
 	/* returns the maximum over all rows */
 	///
-	int getWidthOfColumn(idx_type cell) const;
+	int columnWidth(idx_type cell) const;
 	///
-	int getWidthOfTabular() const;
+	int width() const;
 	///
-	int getAscentOfRow(row_type row) const;
+	int height() const;
 	///
-	int getDescentOfRow(row_type row) const;
+	int rowAscent(row_type row) const;
 	///
-	int getHeightOfTabular() const;
+	int rowDescent(row_type row) const;
 	///
-	void setAscentOfRow(row_type row, int height);
+	void setRowAscent(row_type row, int height);
 	///
-	void setDescentOfRow(row_type row, int height);
+	void setRowDescent(row_type row, int height);
 	///
-	void setWidthOfCell(idx_type cell, int new_width);
+	void setCellWidth(idx_type cell, int new_width);
 	///
 	void setAllLines(idx_type cell, bool line);
 	///
-	void setTopLine(idx_type cell, bool line, bool wholerow = false);
+	void setTopLine(idx_type cell, bool line);
 	///
-	void setBottomLine(idx_type cell, bool line, bool wholerow = false);
+	void setBottomLine(idx_type cell, bool line);
 	///
-	void setLeftLine(idx_type cell, bool line, bool wholecolumn = false);
+	void setLeftLine(idx_type cell, bool line);
 	///
-	void setRightLine(idx_type cell, bool line, bool wholecolumn = false);
+	void setRightLine(idx_type cell, bool line);
 	///
+	bool rowTopLine(row_type row) const;
+	///
+	bool rowBottomLine(row_type row) const;
+	///
+	bool columnLeftLine(col_type column) const;
+	///
+	bool columnRightLine(col_type column) const;
+
 	void setAlignment(idx_type cell, LyXAlignment align,
 			  bool onlycolumn = false);
 	///
@@ -313,27 +316,21 @@ public:
 	///
 	Length const getPWidth(idx_type cell) const;
 	///
-	Length const getColumnPWidth(idx_type cell) const;
-	///
-	Length const getMColumnPWidth(idx_type cell) const;
-	///
-	docstring const getAlignSpecial(idx_type cell, int what) const;
-	///
-	int getWidthOfCell(idx_type cell) const;
+	int cellWidth(idx_type cell) const;
 	///
 	int getBeginningOfTextInCell(idx_type cell) const;
 	///
-	void appendRow(BufferParams const &, idx_type cell);
+	void appendRow(idx_type cell);
 	///
 	void deleteRow(row_type row);
 	///
-	void copyRow(BufferParams const &, row_type);
+	void copyRow(row_type);
 	///
-	void appendColumn(BufferParams const &, idx_type cell);
+	void appendColumn(idx_type cell);
 	///
 	void deleteColumn(col_type column);
 	///
-	void copyColumn(BufferParams const &, col_type);
+	void copyColumn(col_type);
 	///
 	bool isFirstCellInRow(idx_type cell) const;
 	///
@@ -343,19 +340,17 @@ public:
 	///
 	idx_type getLastCellInRow(row_type row) const;
 	///
-	idx_type getNumberOfCells() const;
-	///
 	idx_type numberOfCellsInRow(idx_type cell) const;
 	///
-	void write(Buffer const &, std::ostream &) const;
+	void write(std::ostream &) const;
 	///
-	void read(Buffer const &, Lexer &);
+	void read(Lexer &);
 	///
-	int latex(Buffer const &, odocstream &, OutputParams const &) const;
+	int latex(odocstream &, OutputParams const &) const;
 	//
-	int docbook(Buffer const & buf, odocstream & os, OutputParams const &) const;
+	int docbook(odocstream & os, OutputParams const &) const;
 	///
-	void plaintext(Buffer const &, odocstream &,
+	void plaintext(odocstream &,
 		       OutputParams const & runparams, int const depth,
 		       bool onlydata, char_type delim) const;
 	///
@@ -363,29 +358,17 @@ public:
 	///
 	bool isMultiColumnReal(idx_type cell) const;
 	///
-	void setMultiColumn(Buffer *, idx_type cell, idx_type number);
+	void setMultiColumn(idx_type cell, idx_type number);
 	///
 	idx_type unsetMultiColumn(idx_type cell); // returns number of new cells
 	///
 	bool isPartOfMultiColumn(row_type row, col_type column) const;
 	///
-	row_type row_of_cell(idx_type cell) const;
+	row_type cellRow(idx_type cell) const;
 	///
-	col_type column_of_cell(idx_type cell) const;
+	col_type cellColumn(idx_type cell) const;
 	///
-	col_type right_column_of_cell(idx_type cell) const;
-	///
-	void setBookTabs(bool);
-	///
-	bool useBookTabs() const;
-	///
-	void setLongTabular(bool);
-	///
-	bool isLongTabular() const;
-	///
-	void setRotateTabular(bool);
-	///
-	bool getRotateTabular() const;
+	col_type cellRightColumn(idx_type cell) const;
 	///
 	void setRotateCell(idx_type cell, bool);
 	///
@@ -395,15 +378,11 @@ public:
 	///
 	bool isLastCell(idx_type cell) const;
 	///
-	idx_type getCellAbove(idx_type cell) const;
+	idx_type cellAbove(idx_type cell) const;
 	///
-	idx_type getCellBelow(idx_type cell) const;
+	idx_type cellBelow(idx_type cell) const;
 	///
-	idx_type getLastCellAbove(idx_type cell) const;
-	///
-	idx_type getLastCellBelow(idx_type cell) const;
-	///
-	idx_type getCellNumber(row_type row, col_type column) const;
+	idx_type cellIndex(row_type row, col_type column) const;
 	///
 	void setUsebox(idx_type cell, BoxType);
 	///
@@ -429,6 +408,10 @@ public:
 	///
 	bool getLTNewPage(row_type row) const;
 	///
+	idx_type setLTCaption(row_type row, bool what);
+	///
+	bool ltCaption(row_type row) const;
+	///
 	bool haveLTHead() const;
 	///
 	bool haveLTFirstHead() const;
@@ -439,39 +422,37 @@ public:
 	///
 	// end longtable support
 	///
-	boost::shared_ptr<InsetText> getCellInset(idx_type cell) const;
+	boost::shared_ptr<InsetTableCell> cellInset(idx_type cell) const;
 	///
-	boost::shared_ptr<InsetText> getCellInset(row_type row,
+	boost::shared_ptr<InsetTableCell> cellInset(row_type row,
 						  col_type column) const;
 	///
 	void setCellInset(row_type row, col_type column,
-			  boost::shared_ptr<InsetText>) const;
+			  boost::shared_ptr<InsetTableCell>) const;
 	/// Search for \param inset in the tabular, with the
 	///
-	idx_type getCellFromInset(Inset const * inset) const;
-	///
-	row_type rows() const { return rows_; }
-	///
-	col_type columns() const { return columns_;}
-	///
 	void validate(LaTeXFeatures &) const;
-	///
 //private:
+  // FIXME Now that cells have an InsetTableCell as their insets, rather
+  // than an InsetText, it'd be possible to reverse the relationship here,
+  // so that cell_vector was a vector<InsetTableCell> rather than a 
+  // vector<CellData>, and an InsetTableCell had a CellData as a member,
+  // or perhaps just had its members as members.
 	///
-	class cellstruct {
+	class CellData {
 	public:
 		///
-		cellstruct(BufferParams const &);
+		CellData(Buffer &);
 		///
-		cellstruct(cellstruct const &);
+		CellData(CellData const &);
 		///
-		cellstruct & operator=(cellstruct);
+		CellData & operator=(CellData);
 		///
-		void swap(cellstruct & rhs);
+		void swap(CellData & rhs);
 		///
 		idx_type cellno;
 		///
-		int width_of_cell;
+		int width;
 		///
 		int multicolumn;
 		///
@@ -495,27 +476,23 @@ public:
 		///
 		Length p_width; // this is only set for multicolumn!!!
 		///
-		boost::shared_ptr<InsetText> inset;
+		boost::shared_ptr<InsetTableCell> inset;
 	};
-	cellstruct & cellinfo_of_cell(idx_type cell) const;
+	CellData & cellInfo(idx_type cell) const;
 	///
-	typedef std::vector<cellstruct> cell_vector;
+	typedef std::vector<CellData> cell_vector;
 	///
 	typedef std::vector<cell_vector> cell_vvector;
 
 	///
-	class rowstruct {
+	class RowData {
 	public:
 		///
-		rowstruct();
+		RowData();
 		///
-		int ascent_of_row;
+		int ascent;
 		///
-		int descent_of_row;
-		///
-		bool top_line;
-		///
-		bool bottom_line;
+		int descent;
 		/// Extra space between the top line and this row
 		Length top_space;
 		/// Ignore top_space if true and use the default top space
@@ -537,39 +514,33 @@ public:
 		bool endfoot;
 		/// row of endlastfoot
 		bool endlastfoot;
-		/// row for a pagebreak
+		/// row for a newpage
 		bool newpage;
+		/// caption
+		bool caption;
 	};
 	///
-	typedef std::vector<rowstruct> row_vector;
+	typedef std::vector<RowData> row_vector;
 
 	///
-	class columnstruct {
+	class ColumnData {
 		public:
 		///
-		columnstruct();
+		ColumnData();
 		///
 		LyXAlignment alignment;
 		///
 		VAlignment valignment;
 		///
-		bool left_line;
-		///
-		bool right_line;
-		///
-		int  width_of_column;
+		int width;
 		///
 		Length p_width;
 		///
 		docstring align_special;
 	};
 	///
-	typedef std::vector<columnstruct> column_vector;
+	typedef std::vector<ColumnData> column_vector;
 
-	///
-	row_type rows_;
-	///
-	col_type columns_;
 	///
 	idx_type numberofcells;
 	///
@@ -583,8 +554,6 @@ public:
 	///
 	mutable cell_vvector cell_info;
 	///
-	int width_of_tabular;
-	///
 	bool use_booktabs;
 	///
 	bool rotate;
@@ -602,43 +571,36 @@ public:
 	ltType endlastfoot;
 
 	///
-	void init(BufferParams const &, row_type rows_arg,
+	void init(Buffer &, row_type rows_arg,
 		  col_type columns_arg);
 	///
-	void set_row_column_number_info();
-	/// Returns true if a complete update is necessary, otherwise false
-	bool setWidthOfMulticolCell(idx_type cell, int new_width);
+	void updateIndexes();
 	///
-	void recalculateMulticolumnsOfColumn(col_type column);
-	/// Returns true if change
-	void calculate_width_of_column(col_type column);
+	bool setFixedWidth(row_type r, col_type c);
 	///
-	bool calculate_width_of_column_NMC(col_type column); // no multi cells
+	void updateContentAlignment(row_type r, col_type c);
+	/// return true of update is needed
+	bool updateColumnWidths();
 	///
-	void calculate_width_of_tabular();
-	///
-	void delete_column(col_type column);
-	///
-	idx_type cells_in_multicolumn(idx_type cell) const;
+	idx_type columnSpan(idx_type cell) const;
 	///
 	BoxType useParbox(idx_type cell) const;
 	///
 	// helper function for Latex returns number of newlines
 	///
-	int TeXTopHLine(odocstream &, row_type row) const;
+	int TeXTopHLine(odocstream &, row_type row, std::string const lang) const;
 	///
-	int TeXBottomHLine(odocstream &, row_type row) const;
+	int TeXBottomHLine(odocstream &, row_type row, std::string const lang) const;
 	///
-	int TeXCellPreamble(odocstream &, idx_type cell) const;
+	int TeXCellPreamble(odocstream &, idx_type cell, bool & ismulticol) const;
 	///
-	int TeXCellPostamble(odocstream &, idx_type cell) const;
+	int TeXCellPostamble(odocstream &, idx_type cell, bool ismulticol) const;
 	///
-	int TeXLongtableHeaderFooter(odocstream &, Buffer const & buf,
-				     OutputParams const &) const;
+	int TeXLongtableHeaderFooter(odocstream &, OutputParams const &) const;
 	///
 	bool isValidRow(row_type const row) const;
 	///
-	int TeXRow(odocstream &, row_type const row, Buffer const & buf,
+	int TeXRow(odocstream &, row_type const row,
 		   OutputParams const &) const;
 	///
 	// helper functions for plain text
@@ -649,45 +611,124 @@ public:
 	bool plaintextBottomHLine(odocstream &, row_type row,
 				  std::vector<unsigned int> const &) const;
 	///
-	void plaintextPrintCell(Buffer const &, odocstream &,
+	void plaintextPrintCell(odocstream &,
 				OutputParams const &,
 				idx_type cell, row_type row, col_type column,
 				std::vector<unsigned int> const &,
 				bool onlydata) const;
 	/// auxiliary function for docbook
-	int docbookRow(Buffer const & buf, odocstream & os, row_type,
-		       OutputParams const &) const;
+	int docbookRow(odocstream & os, row_type, OutputParams const &) const;
+
+	/// change associated Buffer
+	void setBuffer(Buffer & buffer);
+	/// retrieve associated Buffer
+	Buffer & buffer() const { return *buffer_; }
 
 private:
-	/// renumber cells after structural changes
-	void fixCellNums();
+	Buffer * buffer_;
+
+}; // Tabular
+
+
+///
+class InsetTableCell : public InsetText
+{
+public:
+	///
+	InsetTableCell(Buffer & buf);
+	///
+	InsetCode lyxCode() const { return CELL_CODE; }
+	///
+	Inset * clone() { return new InsetTableCell(*this); }
+	///
+	bool getStatus(Cursor & cur, FuncRequest const & cmd,
+		FuncStatus & status) const;
+	///
+	void toggleFixedWidth(bool fw) { isFixedWidth = fw; }
+	///
+	void setContentAlignment(LyXAlignment al) {contentAlign = al; }
+	/// writes the contents of the cell as a string, optionally
+	/// descending into insets
+	docstring asString(bool intoInsets = true);
+private:
+	/// unimplemented
+	InsetTableCell();
+	/// unimplemented
+	void operator=(InsetTableCell const &);
+	// FIXME
+	// This boolean is supposed to track whether the cell has had its
+	// width explicitly set. We need to know this to determine whether
+	// layout changes and paragraph customization are allowed---that is,
+	// we need it in forcePlainLayout() and allowParagraphCustomization(). 
+	// Unfortunately, that information is not readily available in 
+	// InsetTableCell. In the case of multicolumn cells, it is present
+	// in CellData, and so would be available here if CellData were to
+	// become a member of InsetTableCell. But in the other case, it isn't
+	// even available there, but is held in Tabular::ColumnData.
+	// So, the present solution uses this boolean to track the information
+	// we need to track, and tries to keep it updated. This is not ideal,
+	// but the other solutions are no better. These are:
+	// (i)  Keep a pointer in InsetTableCell to the table;
+	// (ii) Find the table by iterating over the Buffer's insets.
+	// Solution (i) raises the problem of updating the pointer when an 
+	// InsetTableCell is copied, and we'd therefore need a copy constructor
+	// in InsetTabular and then in Tabular, which seems messy, given how 
+	// complicated those classes are. Solution (ii) involves a lot of 
+	// iterating, since this information is needed quite often, and so may
+	// be quite slow.
+	// So, well, if someone can do better, please do!
+	// --rgh
+	///
+	bool isFixedWidth;
+	// FIXME: Here the thoughts from the comment above also apply.
+	///
+	LyXAlignment contentAlign;
+	/// should paragraph indendation be omitted in any case?
+	bool neverIndent() const { return true; }
+	///
+	LyXAlignment contentAlignment() const { return contentAlign; }
+	///
+	virtual bool usePlainLayout() const { return true; }
+	/// 
+	virtual bool forcePlainLayout(idx_type = 0) const;
+	/// 
+	virtual bool allowParagraphCustomization(idx_type = 0) const;
+	/// Is the width forced to some value?
+	bool hasFixedWidth() const { return isFixedWidth; }
 };
 
 
-
-class InsetTabular : public Inset {
+class InsetTabular : public Inset
+{
 public:
 	///
-	InsetTabular(Buffer const &, row_type rows = 1,
+	InsetTabular(Buffer &, row_type rows = 1,
 		     col_type columns = 1);
 	///
 	~InsetTabular();
 	///
-	void read(Buffer const &, Lexer &);
+	void setBuffer(Buffer & buffer);
+
 	///
-	void write(Buffer const &, std::ostream &) const;
+	static void string2params(std::string const &, InsetTabular &);
 	///
-	bool metrics(MetricsInfo &, Dimension &) const;
+	static std::string params2string(InsetTabular const &);
+	///
+	void read(Lexer &);
+	///
+	void write(std::ostream &) const;
+	///
+	void metrics(MetricsInfo &, Dimension &) const;
 	///
 	void draw(PainterInfo & pi, int x, int y) const;
 	///
 	void drawSelection(PainterInfo & pi, int x, int y) const;
 	///
-	virtual docstring const editMessage() const;
+	docstring editMessage() const;
 	///
 	EDITABLE editable() const { return HIGHLY_EDITABLE; }
 	///
-	bool insetAllowed(Inset::Code) const { return true; }
+	bool insetAllowed(InsetCode code) const;
 	///
 	bool allowSpellCheck() const { return true; }
 	///
@@ -697,20 +738,19 @@ public:
 	    insets that may contain several paragraphs */
 	bool noFontChange() const { return true; }
 	///
-	DisplayType display() const { return tabular.isLongTabular() ? AlignCenter : Inline; }
+	DisplayType display() const { return tabular.is_long_tabular ? AlignCenter : Inline; }
 	///
-	int latex(Buffer const &, odocstream &,
-		  OutputParams const &) const;
+	int latex(odocstream &, OutputParams const &) const;
 	///
-	int plaintext(Buffer const &, odocstream &,
-		      OutputParams const &) const;
+	int plaintext(odocstream &, OutputParams const &) const;
 	///
-	int docbook(Buffer const &, odocstream &,
-		    OutputParams const &) const;
+	int docbook(odocstream &, OutputParams const &) const;
 	///
 	void validate(LaTeXFeatures & features) const;
 	///
-	Code lyxCode() const { return Inset::TABULAR_CODE; }
+	InsetCode lyxCode() const { return TABULAR_CODE; }
+	///
+	docstring contextMenu(BufferView const & bv, int x, int y) const;
 	/// get offset of this cursor slice relative to our upper left corner
 	void cursorPos(BufferView const & bv, CursorSlice const & sl,
 		bool boundary, int & x, int & y) const;
@@ -724,11 +764,11 @@ public:
 	///
 	bool showInsetDialog(BufferView *) const;
 	/// number of cells
-	size_t nargs() const { return tabular.getNumberOfCells(); }
+	size_t nargs() const { return tabular.numberofcells; }
 	///
-	boost::shared_ptr<InsetText const> cell(idx_type) const;
+	boost::shared_ptr<InsetTableCell const> cell(idx_type) const;
 	///
-	boost::shared_ptr<InsetText> cell(idx_type);
+	boost::shared_ptr<InsetTableCell> cell(idx_type);
 	///
 	Text * getText(int) const;
 
@@ -742,40 +782,76 @@ public:
 	// this should return true if we have a "normal" cell, otherwise false.
 	// "normal" means without width set!
 	/// should all paragraphs be output with "Standard" layout?
-	bool forceDefaultParagraphs(idx_type cell = 0) const;
-
+	virtual bool allowParagraphCustomization(idx_type cell = 0) const;
+	///
+	virtual bool forcePlainLayout(idx_type cell = 0) const;
+	///
+	virtual bool usePlainLayout() { return true; }
 	///
 	void addPreview(graphics::PreviewLoader &) const;
 
-	///
-	Buffer const & buffer() const;
-
-	/// set the owning buffer
-	void buffer(Buffer const * buf);
 	/// lock cell with given index
-	void edit(Cursor & cur, bool left);
+	void edit(Cursor & cur, bool front, EntryDirection entry_from);
+	/// get table row from x coordinate
+	int rowFromY(Cursor & cur, int y) const;
+	/// get table column from y coordinate
+	int columnFromX(Cursor & cur, int x) const;
 	///
 	Inset * editXY(Cursor & cur, int x, int y);
 	/// can we go further down on mouse click?
 	bool descendable() const { return true; }
+	// Update the counters of this inset and of its contents
+	void updateLabels(ParIterator const &);
 
+	///
+	bool completionSupported(Cursor const &) const;
+	///
+	bool inlineCompletionSupported(Cursor const & cur) const;
+	///
+	bool automaticInlineCompletion() const;
+	///
+	bool automaticPopupCompletion() const;
+	///
+	bool showCompletionCursor() const;
+	///
+	CompletionList const * createCompletionList(Cursor const & cur) const;
+	///
+	docstring completionPrefix(Cursor const & cur) const;
+	///
+	bool insertCompletion(Cursor & cur, docstring const & s, bool finished);
+	///
+	void completionPosAndDim(Cursor const &, int & x, int & y, Dimension & dim) const;
+	///
+	virtual bool usePlainLayout() const { return true; }
+
+	///
+	virtual InsetTabular * asInsetTabular() { return this; }
+	///
+	virtual InsetTabular const * asInsetTabular() const { return this; }
+	///
+	bool isRightToLeft(Cursor & cur) const;
+	/// writes the cells between stidx and enidx as a string, optionally
+	/// descending into the insets
+	docstring asString(idx_type stidx, idx_type enidx, bool intoInsets = true);
+
+	/// Returns whether the cell in the specified row and column is selected.
+	bool isCellSelected(Cursor & cur, row_type row, col_type col) const;
 	//
 	// Public structures and variables
 	///
 	mutable Tabular tabular;
 
-protected:
+private:
 	///
 	InsetTabular(InsetTabular const &);
 	///
-	virtual void doDispatch(Cursor & cur, FuncRequest & cmd);
+	void doDispatch(Cursor & cur, FuncRequest & cmd);
 	///
 	bool getStatus(Cursor & cur, FuncRequest const & cmd, FuncStatus &) const;
 	///
 	int scroll() const { return scx_; }
-
-private:
-	virtual std::auto_ptr<Inset> doClone() const;
+	///
+	Inset * clone() const { return new InsetTabular(*this); }
 
 	///
 	void drawCellLines(frontend::Painter &, int x, int y, row_type row,
@@ -784,11 +860,13 @@ private:
 	void setCursorFromCoordinates(Cursor & cur, int x, int y) const;
 
 	///
-	void moveNextCell(Cursor & cur);
+	void moveNextCell(Cursor & cur, 
+				EntryDirection entry_from = ENTRY_DIRECTION_IGNORE);
 	///
-	void movePrevCell(Cursor & cur);
+	void movePrevCell(Cursor & cur,
+				EntryDirection entry_from = ENTRY_DIRECTION_IGNORE);
 	///
-	int getCellXPos(idx_type cell) const;
+	int cellXPos(idx_type cell) const;
 	///
 	void resetPos(Cursor & cur) const;
 	///
@@ -800,14 +878,10 @@ private:
 	///
 	void cutSelection(Cursor & cur);
 	///
-	bool isRightToLeft(Cursor & cur) const;
-	///
 	void getSelection(Cursor & cur, row_type & rs, row_type & re,
 			  col_type & cs, col_type & ce) const;
 	///
 	bool insertPlaintextString(BufferView &, docstring const & buf, bool usePaste);
-	/// are we operating on several cells?
-	bool tablemode(Cursor & cur) const;
 
 	/// return the "Manhattan distance" to nearest corner
 	int dist(BufferView &, idx_type cell, int x, int y) const;
@@ -819,37 +893,17 @@ private:
 				row_type row_start, row_type row_end,
 				col_type col_start, col_type col_end) const;
 	///
-	Buffer const * buffer_;
-	///
 	mutable idx_type first_visible_cell;
 	///
 	mutable int scx_;
-};
-
-
-class InsetTabularMailer : public MailInset {
-public:
-	///
-	InsetTabularMailer(InsetTabular const & inset);
-	///
-	virtual Inset & inset() const { return inset_; }
-	///
-	virtual std::string const & name() const { return name_; }
-	///
-	virtual std::string const inset2string(Buffer const &) const;
-	///
-	static void string2params(std::string const &, InsetTabular &);
-	///
-	static std::string const params2string(InsetTabular const &);
-private:
-	///
-	static std::string const name_;
-	///
-	InsetTabular & inset_;
+	/// true when selecting rows with the mouse
+	bool rowselect_;
+	/// true when selecting columns with the mouse
+	bool colselect_;
 };
 
 std::string const featureAsString(Tabular::Feature feature);
 
 } // namespace lyx
 
-#endif
+#endif // INSET_TABULAR_H
