@@ -66,6 +66,7 @@ class InsetMathNest;
 class InsetMathScript;
 class InsetMathString;
 class InsetMathSpace;
+class InsetMathSpecialChar;
 class InsetMathSymbol;
 class InsetMathUnknown;
 
@@ -112,6 +113,7 @@ public:
 	virtual InsetMathAMSArray const * asAMSArrayInset() const { return 0; }
 	virtual InsetMathArray          * asArrayInset()          { return 0; }
 	virtual InsetMathArray const    * asArrayInset() const    { return 0; }
+	virtual InsetMathBrace          * asBraceInset()          { return 0; }
 	virtual InsetMathBrace const    * asBraceInset() const    { return 0; }
 	virtual InsetMathChar const     * asCharInset() const     { return 0; }
 	virtual InsetMathDelim          * asDelimInset()          { return 0; }
@@ -141,6 +143,7 @@ public:
 	virtual InsetMathUnknown        * asUnknownInset()        { return 0; }
 	virtual InsetMathUnknown const  * asUnknownInset() const  { return 0; }
 	virtual InsetMathRef            * asRefInset()            { return 0; }
+	virtual InsetMathSpecialChar const * asSpecialCharInset() const { return 0; }
 
 	/// identifies things that can get scripts
 	virtual bool isScriptable() const { return false; }
@@ -181,8 +184,7 @@ public:
 	virtual void octave(OctaveStream &) const;
 
 	/// plain text output in ucs4 encoding
-	int plaintext(Buffer const &, odocstream &,
-		      OutputParams const &) const;
+	int plaintext(odocstream &, OutputParams const &) const;
 
 	/// dump content to stderr for debugging
 	virtual void dump() const;
@@ -198,7 +200,7 @@ public:
 	virtual bool allowedIn(mode_type mode) const { return mode == MATH_MODE; }
 
 	/// superscript kerning
-	virtual int kerning() const { return 0; }
+	virtual int kerning(BufferView const *) const { return 0; }
 };
 
 ///
