@@ -32,7 +32,7 @@ public:
 	///
 	void incSpace();
 	///
-	bool metrics(MetricsInfo & mi, Dimension & dim) const;
+	void metrics(MetricsInfo & mi, Dimension & dim) const;
 	///
 	void draw(PainterInfo & pi, int x, int y) const;
 
@@ -49,9 +49,11 @@ public:
 	///
 	void write(WriteStream & os) const;
 private:
-	virtual std::auto_ptr<Inset> doClone() const;
+	virtual Inset * clone() const;
 	///
 	int space_;
+	///
+	Dimension dim_;
 };
 
 
