@@ -12,8 +12,7 @@
 #ifndef MATH_EXTERN_H
 #define MATH_EXTERN_H
 
-#include "support/docstring.h"
-#include <string>
+#include "support/strfwd.h"
 
 namespace lyx {
 
