@@ -15,19 +15,10 @@
 #include <iostream>
 #include <sstream>
 
+using namespace std;
 
 namespace lyx {
 
-using std::cerr;
-using std::endl;
-using std::fill;
-using std::istream;
-using std::istringstream;
-using std::ostringstream;
-using std::ostream;
-using std::string;
-
-
 namespace {
 
 CatCode theCatcode[256];
@@ -282,14 +273,14 @@ Parser::Arg Parser::getFullArg(char left, char right)
 	// This is needed if a partial file ends with a command without arguments,
 	// e. g. \medskip
 	if (! good())
-		return std::make_pair(false, string());
+		return make_pair(false, string());
 
 	string result;
 	char c = getChar();
 
 	if (c != left) {
 		putback();
-		return std::make_pair(false, string());
+		return make_pair(false, string());
 	} else
 		while ((c = getChar()) != right && good()) {
 			// Ignore comments
@@ -301,7 +292,7 @@ Parser::Arg Parser::getFullArg(char left, char right)
 				result += curr_token().asInput();
 		}
 
-	return std::make_pair(true, result);
+	return make_pair(true, result);
 }
 
 
@@ -316,7 +307,7 @@ string Parser::getFullOpt()
 	Arg arg = getFullArg('[', ']');
 	if (arg.first)
 		return '[' + arg.second + ']';
-	return arg.second;
+	return string();
 }
 
 
@@ -326,14 +317,16 @@ string Parser::getOpt()
 	return res.empty() ? string() : '[' + res + ']';
 }
 
-string Parser::getFullParentheseOpt()
+
+string Parser::getFullParentheseArg()
 {
 	Arg arg = getFullArg('(', ')');
 	if (arg.first)
 		return '(' + arg.second + ')';
-	return arg.second;
+	return string();
 }
 
+
 string const Parser::verbatimEnvironment(string const & name)
 {
 	if (!good())
